local hand_cursor, grey_cursor, help_cursor in _ENV
local format in string

function _G.get_ground_unit_icon(building_ground_unit)
    local icon_name = building_ground_unit
    if building_ground_unit ~='android'
      icon_name..='s'
    end
    return icon_name
end


UI_File | function(_ENV)

depends 'ChooseFrame'
depends 'ImprovementDrawers'
depends 'terraforming'

local get_ground_unit_icon in _G

local function opt_progress_bar(active,bar0,bar1,t)

  if  ( active or ( (bar0 or 0) >0) )
    bar0=bar0 or 0
    t.hover_text..=format('\n\nConstruction Progress: %d%%',round|bar0*100)
    t.progress_bar = {
      ..bar0,
      bar1= active and bar1,
      colors=  'oranges',
    }
  end
  return t
end

local frame = ChooseSkin1.NewFrame(_ENV,|) {
  rect = function ()
    local layout_h in layout
    local top_bar in _ENV
    local bar_x= planet_bar.x
    local mx,my,mw,mh = midscreen_rect()
    
    --local s=250
    --local xsep=14

    local x,y=mx+10,bottom_bar.height+22

    local w= bar_x-24-mx

    x-=7
    w+=8

    local h= layout_h-bottom_bar.height-top_bar.height-86


    return x,y,w,h
  end,
  min_xpad=4, xsep=12,
}



local unbuildable = {}



function is_planet_close_element(el)
  local on_click in el?
  return (on_click == planet_pane.close_choose) or (on_click==close_midscreen_pane)
end


local function clear_layout()
  draw_candidates=nil
  planet_pane.limit_production_candidates=nil
end

insert(planet_pane,..clear_layout)

planet_pane.limit_production_candidates=nil

function set_production_limit(el)
  clear_layout()
  planet_pane.limit_production_candidates=el.limit

end

local function new_design_click(el)
  open_new_design_screen('New')
end

make_ship_hover_stats = function(ship)
  local text, pushf,catf,push in new_text_pusher()

  local speed = ship.empire : speed_from_class( ship.speed_class )  
  local c1 = add_align(5)
  local c2 = add_align(10)
  local c3 = add_align(12)

  pushf('%d<HULL> %s%d<ARMOR>',ship.structure, c1, ship.armor)

  --if ship.max_shields > 0
    catf(' %s%d<SHIELD>',c2,ship.max_shields,)
  --else
  --  catf('%s',c3)
 -- end

  pushf('%d<SPEED>',speed,c1)

  catf('%s%d<MARINES>',c1,ship.max_crew+ship.crew_from_embarked)

  local coins = SCOPE.trade_item?.coin_upkeep or ship.coin_upkeep
  local drawer_table in ship
  if drawer_table.fixed_defense and not drawer_table.mobile_station
    coins=0
  end

  catf('%s%d<COIN>',c2,coins)

  --[[
  if (ship.munitions or 0) >0
    catf('%s%d<AMMO>',c2,ship.munitions)
  end
  --]]

  return shorthand_format | text | vskip_newline(4)
end

make_ship_hover_text = function(ship,invalid,planet)
  
  local config =  ship.drawer_table.configs[ship.config_name]
  
  local text, pushf,catf,push, lines in new_text_pusher()

  if invalid
    if invalid=='construction_tier'
      
      local required_tier in config!
      local construction_tier in planet!

      push | desc_format | format(
      [=[
      Requires a planet with <b>%d</b> construction module%s in orbit.
      This planet has <b>%d</b> construction modules.
      ]=], required_tier, required_tier==1 and '' or 's', construction_tier
      )

      pushf ''
    elseif invalid == 'mercenary_cant_refit'

      pushf("mercenary units may never be refitted.",id_to_message | invalid)
      pushf ''

    elseif invalid=='insufficient_pop'
      push("<PLANET_NAME> lacks the population to equip a colony ship.")
			
			local race = get_colonist_species (planet)
			if planet.pop[race]
				local delta = 2-planet.pop[race]
				
				catf(" %s additional <bi>%s colonists</bi> are required before we can outfit the vessel.",
					pop_string| delta,race_string_Plural | race)
			else
				catf(" None of the populations currently on the planet would serve as effective colonists.")
			end
			
      pushf ''
    elseif invalid=='already_refitting'

      pushf("a refit action is already in progress.",ship.name)
      pushf ''
    elseif invalid=='cant_change_production'

      pushf("because <PLANET_NAME>'s production cannot be changed until next turn.",ship.name)
      pushf ''

    elseif invalid=='hull_locked'

      pushf("because we are unfamiliar with the technology used in its construction.",ship.name)
      pushf ''


    else
      pushf("<b>%s</b>",id_to_message | invalid)
      pushf ''
    end
  end


  push | make_ship_hover_stats | ship

  if ship.colony
    pushf ''
    pushf("Can found a colony on uninhabited worlds.")
  end

  if ship.source_design.carried_design
    pushf ''
    pushf("Can place a station in orbit around an uninhabited world.")
  end

  if ship.scanner
    pushf ''
    pushf("Able to discover hidden resources, and detect alien ships at greater distances.")
  end

  if (ship.beaker_gen or 0) > 0
    pushf ''
    pushf("Generates +%d<BEAKER> each turn.",ship.beaker_gen)
  end

  if ship.trade_capacity > 0
    pushf ''
    pushf('Provides %d<CARGO> if assigned to the trade pool.  <CARGO> can be used to move <FOOD> between planets, or generate <COIN> through trade.',ship.trade_capacity)
  end

  if (ship.construction_tier or 0) > 0
    pushf ''
    pushf('Increases the capacity of the planet to build large starships by +%d hull tier%s.',ship.construction_tier, ship.construction_tier ==1 and '' or 's')   
  end

  if ship.drawer_table.fixed_defense
    pushf ''
    pushf "Bases deployed in orbit around a colony world cost no upkeep."
  end

  local stored0 in SCOPE
  if stored0 
    pushf ''
    pushf( 'Construction Progress: %d%%', round | stored0 * 100 )
  end

  return shorthand_format | text('\n')
end

design_to_ship = weak_memoizeNN | function(design)
  local rval = EmulateRecorded.all_ships | design.genShip()

	if design.carried_design
		local carried_ship = EmulateRecorded.all_ships | design.carried_design.genShip()
		rval.carried_ship = carried_ship
		carried_ship.carrying_ship=rval
	end

  return rval
end 

local function ground_unit_candidates(push_main)
  
  local planet=selected_planet
  local empire in planet
  local show_metal_warning in planet

  local function opt_bar(t)
    local bar0,bar1=planet : stored_progress_for__building_ground_unit(t!.building_ground_unit)
    local active = planet.building_ground_unit==t.building_ground_unit

    opt_progress_bar(active,bar0,bar1,t)
    
    return t
  end

  push_main | opt_bar {
    building_ground_unit = 'tank',
    name = planet.tank_name,
    costs = catch | planet : tank_cost(),

      hover_text = desc_format | format( [=[

        Strength %d<COMBAT_POW> ground units.  Maintinance requires
        %d<COIN> each turn.
        
        <br><br> If carried by Transports to hostile planets, %s can be used to
        perform <bi>Invasion</bi> actions.
        
        <br><br>

        On planets with discontent <DISCONTENT>, the presence of
        ground units will suppress rebellions.


        ]=], planet.empire.armor_rating, planet.empire.tank_upkeep_cost, planet.tank_name )

  }

  if planet:can_build_ground_unit 'marauder_raider'
    push_main | opt_bar {
      building_ground_unit = 'marauder_raider',
      name = 'Marauder Raiders',
      costs = catch | planet : raider_cost(),
      hover_text = desc_format | format( [=[

        Strength %g<COMBAT_POW> ground units.  Maintenance requires
        %d<COIN> each turn. 
        
        <br><br> If carried by Transports to hostile planets, Marauder
        Raiders can be used in both <bi>Invasion</bi> and <bi>Raid</bi>
        actions. Marauder <bi>raids</bi> have a chance of capturing enemy
        civilians as <bi>slaves</bi>.

        <br><br> Each Raider's <COMBAT_POW> counts double for the purposes of
        suppressing rebellions. 
        
        ]=], planet.empire:raider_rating 'marauder_raiders', planet:upkeep_per_ground_unit 'marauder_raider' )
        
  } 
 end

  if planet:can_build_ground_unit 'pirate_raider'
    push_main | opt_bar {
      building_ground_unit = 'pirate_raider',
      name = 'Pirate Raiders',
      costs = catch | planet : raider_cost(),
    }
  end


  if planet:can_build_ground_unit 'armored_raider'
    push_main | opt_bar {
      building_ground_unit = 'armored_raider',
      name = 'Assault Marines',
      costs = catch | planet : raider_cost(),
      hover_text = desc_format | format([=[
        Strength %g<COMBAT_POW> ground units.  Maintenance requires
        %d<COIN> each turn. 

        <br><br> Like other ground units, if carried by Transports to hostile
        planets, Assault Marines can be used to perform <bi>Invasion</bi>
        actions. <br><br>
                    
        Assault Marines increase the <MARINES> rating of any ship carrying
        them by <b>+%d</b>. 
      ]=], planet.empire:raider_rating 'armored_raiders', planet:upkeep_per_ground_unit 'armored_raider', planet.empire:crew_per_embarked('armored_raiders'))

    }
  end

  if planet:can_build_ground_unit 'android'
    push_main | opt_bar {
      building_ground_unit = 'android',
      name = 'Androids',
      costs = catch | planet:android_cost(),
      hover_text = desc_format | format([=[
        Artificial workers.  They consume no <FOOD>, and produce no <BEAKER>
        or <COIN>, but are very effective at producing labor (<WRENCH>).
      ]=])

    }
  end



end

local function ship_candidates(push_main)
  local planet=selected_planet
  local empire in planet
  local push, ship_candidates = pusher {}

  local show_metal_warning in planet

  for design_key,design ; empire.designs

    if not design.dont_list_in_choose_production

    local ship = design_to_ship | design

    local stored0, stored1 = planet : stored_progress_for__building_design(design_key) 

    local invalid = planet:cant_build_design(design_key)

    local active = (design_key==planet.building_design) 
    if active
      stored0=stored0 or 0
    end

		if invalid=='no_debris_ring'
			goto next_design
		end

    push {
      name=design!.design_name,
      ..ship,
      ..design,
      building_design=design_key,
      ..invalid,
      costs = catch | planet:design_cost(design_key),
      progress_bar = ( (( stored0 or 0 )> 0) or active ) and {
        bar0=stored0,
        bar1= active and stored1,
        colors= (show_metal_warning and active and 'reds') or 'oranges',
      },
      hover_text = SCOPE_call(..stored0,..stored1,
        make_ship_hover_text,ship,invalid,selected_planet),
    }

    end

		::next_design::

  end

  table.sort(ship_candidates,function(c1,c2)

    local f1 = not c1.design.fixed_defense
    local f2 = not c2.design.fixed_defense

    if(f1 and not f2) 
      return true 
    elseif(f2 and not f1) 
      return false
    end

    return (c1.design._ikey or 0)	< (c2.design._ikey or 0)

  end)

  for i,v in ipairs(ship_candidates)
    push_main | v
  end

end


local function working_refit_candidates(push)
  
  local planet = selected_planet

  for ship ; planet.refits?
    local function on_click(el)
      FLAG_call('player_set_production', order.continue_refitting,planet,ship)
    end

    local t= {
      name=refit_project_name(ship),
      --name = production_name(project),
      refitting=ship,
      ship=ship,
      ..on_click,
      costs = catch | planet:refit_cost(ship),
      hover_text = format('Continue refitting %s.\n',ship.name) 
          .. make_ship_hover_text(ship,false,selected_planet)
    }

    local bar0,bar1=planet : stored_progress_for__refitting(ship)
    local active=planet.refitting==ship
    opt_progress_bar(active,bar0,bar1,t)
    push | t
    
  end	 
end

local function open_refit_subpane(el)
  planet_subpane='Refit'
  clear_layout()
end

local function open_refit_subpane_with_queue(el)
  planet_subpane='Refit Queue'
  clear_layout()
  print('doing stuffs')
end


local function improvement_candidates(push)
  local planet = selected_planet 

  local empire = gui_player()

  local possible_improvements = empire:possible_improvements(planet)

  for i, name in ipairs {'Machine Altar', 'Factory','Market','Mine','Farm','Lab','Planetary Defenses'}

    if possible_improvements[name]
      local drawfun = ImprovementDrawers[name] or _SAFE
      local invalid = planet:cant_build_improvement(name)

      local bar0,bar1 = planet:stored_progress_for__building_improvement(name)


      local t= { 
        ..name, ..drawfun, ..invalid,  
        costs = catch | planet:improvement_cost(name),
        project=name,building_improvement=name,

        hover_text = CHOOSE_HOVER[name]?(planet),
      }

      local active = planet.building_improvement==name

      opt_progress_bar(active,bar0,bar1,t)

      push | t

    end

  end

end


local function refit_click(el)
  local candidate in el!
  local ship in candidate
  local planet = selected_planet
  local add_to_queue in el
  add_to_queue=add_to_queue or (planet_subpane=='Refit Queue')
  local design = Design.clone(ship.carried_ship?.source_design or ship!.source_design)

  design.empire=gui_player()

  open_designer {
    mode='Refit',
    ..design,

    on_commit = FLAG_fun('player_set_production',|)function(new_design)
      
      --new_design

      console('starting ship refit on',planet)
      console(..t2s | new_design)

      local fleet_i, fleet, orbit_i in candidate

      clear_layout()
      planet_subpane='choose_production'

      local old_project,old_project_info = planet:project_info()

      execute_order('star_refit',|) function()

        order.start_refit(planet,|) {
          ..ship,
          ..orbit_i,
          ..fleet,
          ..new_design,
        }

        if add_to_queue and old_project
          order.set_planet_project(  planet, {[old_project]=old_project_info} )
          order.enqueue_project(planet, {refitting=ship})
          planet_pane.show_queue =true
        end
      end

    end
  }
  designer_pane.call_on_close = nil
end

local function refit_option(push)
  local planet=selected_planet
  local empire,star in planet

  local push_ship, refit_candidates = pusher {}

  local function do_ship(ship,fleet,orbit_i)


        local invalid = planet:cant_refit(ship) 
        --can_refit(planet)

        if planet.fixed_production and not invalid
          invalid='cant_change_production'
        end

        local options = get_ship_refit_info(ship)
        local refit_option = options?[1]

        local instant_refit_text = "Can't instant refit"
        local instant_refit_invalid=true
        local mass_refit_info

        if refit_option

          if refit_option.ident

            instant_refit_text = 
               shorthand_format | format(
               "Already up to date with our current <i>%s</i> design.",
               refit_option.design.design_name)
          else
            local coin_cost,metal_cost in refit_option
            instant_refit_invalid= (coin_cost>ship.empire.credits) or (metal_cost>ship.empire.metal)
            instant_refit_text = 
               shorthand_format | format(
                'Refit instantly to the current <i>%s</i> design for %d<COIN> and %d<METAL>.',
                refit_option.design.design_name,
                refit_option.coin_cost,
                refit_option.metal_cost,
              )

            local mass_ships,refitable = get_class_refit_info(ship)

            if refitable>1

              local invalid =  (refitable*refit_option.coin_cost > empire.credits) or
                       (refitable*refit_option.metal_cost > empire.metal)

              local function on_click()
                execute_order('mass_refit',|) function()
                  for i,info in ipairs | mass_ships
                    local ship,cant_instant_refit in info
                    if not cant_instant_refit
                      refit_ship_to_option(ship,refit_option)
                    end
                  end
                end
              end

              local text= shorthand_format | format(
                'Refit all %d %s with this design to the current <i>%s</i> design for a one-time cost of %d<COIN> and %d<METAL>.',
                refitable,
                ship.fixed_defense and 'stations' or 'ships',
                refit_option.design.design_name,
                refit_option.coin_cost*refitable,
                refit_option.metal_cost*refitable
                )

              mass_refit_info={
                ..mass_ships,..refitable,
                ..on_click,..text,
                ..invalid
                }

            end


          end

        end

        push_ship {
          ..ship,..fleet,fleet_i=fleet?._ikey, name = 'Refit: '..ship.name, 
          on_click= (not invalid) and refit_click,
          on_queue_click= (not invalid) and refit_click,
          ..invalid,
          ..orbit_i,
          ..instant_refit_text,
          ..instant_refit_invalid,
          ..refit_option,
          ..mass_refit_info,
          refit_queue=true,


          hover_text =  invalid and 
                          format("Can't refit %s, ",ship.name)..make_ship_hover_text(ship,invalid,selected_planet) ,

          --[[
          hover_text =( invalid and 
                          format("Can't refit %s, ",ship.name) or
                          format('Start refitting %s.\n',ship.name) 
                     ).. make_ship_hover_text(ship,invalid,selected_planet)
  
  --]]

        }

  end

  for _,fleet ; empire.fleets
    if at_star(fleet)==star
      for ship in rpairs | fleet.ships
        
        do_ship(ship,fleet)

        --end
      end
    end
  end

  for orbit_i,ship ; planet.orbitals?

          do_ship(ship,nil,orbit_i)


  end

  if # refit_candidates > 0
    push {
      name='Refit', 
      special_item="Refit",
      on_click=open_refit_subpane,
      on_queue_click=open_refit_subpane_with_queue,
      hover_text = format('Refit one of the ships at %s',selected_planet.star.name),
      }

    return refit_candidates
  end

  

end


local function activities(push)
  local planet = selected_planet

  for name ; planet_activity
    
    local bigtext,small_name

    if not ActivityArt[name]
      bigtext=name
    else
      small_name=name
    end

    if not (gui_player():missing_activity_tech(name) or gui_player():cant_use_activity(name) )

			if not planet:dont_show_activity(name)

				push {
					name = small_name,
					..bigtext,
					activity=name, invalid = planet:cant_set_activity(name),hover_text=CHOOSE_HOVER[name]?(planet) }

			end
 
    end
  end

end

local function terraforming(push)
  local planet = selected_planet

  for target_type, costs ; planet:terraforming_options()
    push {
      --bigtext=terraforming_project_name(target_type,'\n'), 
      ..costs, 
      hover_text=make_terraforming_hover_text(planet,target_type),
      terraforming = target_type, 
      name = terraforming_project_name(target_type,' '), 
    }
  end
end

local function colony_bases(push)
  local star in selected_planet
  local empire in selected_planet

  for i,planet in star:planet_list()
    if empire:can_colonize(planet)
      
      local active = selected_planet.building_colony==planet
      local bar0,bar1 = selected_planet: stored_progress_for__building_colony(planet)

      local t = {
         costs = catch | selected_planet:colony_base_cost(planet), 
         on_click = start_colony_base,
         building_colony=planet,
         name = 'Colonize: '..planet.name,
         hover_text = format('Dispatch colonists and supplies to establish a new colony on <bi>%s</bi>',planet.name),
      }

      opt_progress_bar(active,bar0,bar1,t)

      push | t

    end
  end



  if ( empire:known  'Planet Construction'  ) 

      local a,b=StarOrbitRange![star.type]()

      local typestr = {
        ['White Dwarf']='<bi>White Dwarf</bi> star',
        ['M Red']='<bi>Red Dwarf</bi> star',
 
        ['K Orange']='<bi>K-type Orange</bi> star',
        ['G Yellow']='<bi>G-type Yellow</bi> star',

        ['Red Giant']='<bi>Red Giant</bi>',
        ['AB Blue White']='<bi>Blue Giant</bi>', 
  
      }

      local hover_text = compress_spacing | format([[

        Construct a artificial planet in the %s system. %s is a %s, and can thus
        support up to <b>%d planets</b>.
        
        ]],
        
        star.name,star.name,typestr![star.type],b-a+1
        )
  
      local rem = star.num_planet_build_sites
      if rem== 0
        hover_text..= '\n\nThere are no viable planet construction sites remaining in the system.'
      else 
        hover_text..= format('\n\nThere %s <b>%d</b> planet construction site remaining in the system.',rem==1 and 'is' or 'are',rem)
      end

      local invalid = selected_planet.star.cant_build_more_planets and 'No Build Sites'
      local t= {
         costs = catch | selected_planet:build_planet_cost(), 
         building_planet=true,
         name = 'Artificial Planet',
         ..invalid,
         ..hover_text,
      }

      local active = selected_planet.building_planet
      local bar0,bar1= selected_planet:stored_progress_for__building_planet()

      if not invalid
        opt_progress_bar(active,bar0,bar1,t)
      end

      push | t
  end

end
-- jem.habitat.base.start
local function habitat_bases(push)
  local star in selected_planet
  local empire in selected_planet

  if ( empire:known  'Planet Construction'  ) 

      local a,b=StarOrbitRange![star.type]()

      local typestr = {
        ['White Dwarf']='<bi>White Dwarf</bi> star',
        ['M Red']='<bi>Red Dwarf</bi> star',
 
        ['K Orange']='<bi>K-type Orange</bi> star',
        ['G Yellow']='<bi>G-type Yellow</bi> star',

        ['Red Giant']='<bi>Red Giant</bi>',
        ['AB Blue White']='<bi>Blue Giant</bi>', 
  
      }

      local hover_text = compress_spacing | format([[

        Construct a artificial planet in the %s system. %s is a %s, and can thus
        support up to <b>%d planets</b>.
        
        ]],
        
        star.name,star.name,typestr![star.type],b-a+1
        )
  
      local rem = star.num_planet_build_sites
      if rem== 0
        hover_text..= '\n\nThere are no viable planet construction sites remaining in the system.'
      else 
        hover_text..= format('\n\nThere %s <b>%d</b> planet construction site remaining in the system.',rem==1 and 'is' or 'are',rem)
      end

      local invalid = selected_planet.star.cant_build_more_planets and 'No Build Sites'
      local t= {
         costs = catch | selected_planet:build_planet_cost(), 
         building_planet=true,
         name = 'Artificial Planet',
         ..invalid,
         ..hover_text,
      }

      local active = selected_planet.building_planet
      local bar0,bar1= selected_planet:stored_progress_for__building_planet()

      if not invalid
        opt_progress_bar(active,bar0,bar1,t)
      end

      push | t
  end

end
-- jem.habitat.base.end


function calc_production_candidates(limit)

  local planet = selected_planet
  local empire,star in planet
	local no_colony in planet
  local push, production_candidates = pusher {}

 
  if limit~='projects' and not no_colony
    working_refit_candidates(push)
  end


  if limit~='ships'  and not no_colony
    improvement_candidates(push)
  end

  local refit_candidates = refit_option(push)
--jem.habitat
--  if (limit~='ships')  and no_colony
  if (limit~='ships')  and no_colony or planet.summed_pop==0 --sum(planet.pop?)==0
    habitat_bases(push)
  end
--jem.habitat.end
  if (limit~='ships')  and not no_colony
    terraforming(push)
    colony_bases(push)
    ground_unit_candidates(push)
  end
  if limit~='projects'  and not no_colony
    ship_candidates(push)
  end
  if limit~='ships' 
    activities (push)
  end



  if limit~='ships'  and not no_colony
    push { 
      no_queue_button=true,
      --bigtext = 'New Design', 
      name = 'New Ship Design',
      on_click=new_design_click,
      special_item = 'New Design',

      hover_text = desc_format [[
      
      Create a new <bi>Ship Design</bi>.     

      ]], 
      
    } end

  local repeat_color = selected_planet.cant_repeat and catch(1,1,1,.25) or white

  local repeat_click = function()
      --print'yo'

      --print(..planet.cant_repeat)

      if not planet.cant_repeat
        order.enqueue_project(planet,{repeat_prior=true}) 
        planet_pane.show_queue=true
      end
    end

  push {
    name='Repeat Build',  
    no_queue_button=true,
    outlined_bk=true,
    invalid = planet.cant_repeat,


      hover_text = desc_format [[
      
      Add an item to the production queue that will cause the prior
      production item to be infinitely repeated.

      ]], 

    on_queue_click=repeat_click, --allow right-click to set repeat.
    on_click = repeat_click,
    drawfun=restore_color | function(rx,ry,rw,rh)
      --glBox(rx,ry,rw,rw)

      if selected_planet.cant_repeat
        --print('hrm..')
        set_frame_color4(1,1,1,.25)
      end

      SCOPE_call(
        icon_radius_hint=60,
        icon_color=repeat_color,
        notification_icons.misc.infinity.draw_shadowed_icon,
        rx+rw*.5,ry+rw*.5
      )

      return rx,ry,rw,rw
    end,
  } 

 

  if limit~='ships'
    local invalid = planet.fixed_production and 'PRODUCTION FIXED'
    local icon_color = invalid and catch(.5,.5,.5,1) or white
    push {
      set_autobuild = true,
      name = 'Automate*',
      no_queue_button=true,
      outlined_bk=true,
      ..invalid,

      hover_text = desc_format [[
        Instruct <PLAYER_ADJUTANT_NAME> to manage <PLANET_NAME>
        <bi>automatically</bi>. <br><br>

        Choices made by your adjutant are indicated with '*'.

      ]],

      drawfun=restore_color | function(rx,ry,rw,rh)
        --glBox(rx,ry,rw,rw)

        SCOPE_call(
          icon_radius_hint=70,
          ..icon_color,
          notification_icons.misc.automate.draw_shadowed_icon,
          rx+rw*.5,ry+rw*.5
        )

        return rx,ry,rw,rw
      end,


    } 
  end


  return {
    ..production_candidates, 
    ..refit_candidates,
  }
end

  --[=[
local function change_production( production)

  planet_subpane=nil
  local last_order,empire,args = last_order_info()



  -- trigger in an implicit undo if our last order was to change
  -- the production on this planet
  if(last_order=="set_production" and empire==gui_player() )

    local planet_name=args:match( "..(.*)%._ref")

    if(planet_name==recorded_name(selected_planet))
      undo_order()
    end

  end



  order.set_production( selected_planet, production)
end

--]=]

local function set_auto(el)
  local planet = selected_planet
  order.set_autobuild(planet)
end


function _G.set_project_sound(project)
 
  local building_ground_unit,building_design,building_planet,activity,refit,terraforming,building_improvement,building_colony in project

  Sounds.SELECT_FOR_PRODUCTION()
end

local function set_planet_project(el)


  local building_planet,building_ground_unit,building_design,activity,refit,terraforming,invalid,building_improvement,building_colony in el.candidate
  local planet = selected_planet

  if invalid
    Sounds.INVALID_ACTION()
    return
  end

  set_project_sound(el.candidate)


  if building_design
    undo_set_project(planet)
    order.set_building_design(planet,building_design)
  elseif building_ground_unit
    undo_set_project(planet)
    order.set_building_ground_unit(selected_planet,building_ground_unit)
  elseif building_colony
    undo_set_project(planet)
    order.set_building_colony(selected_planet,building_colony)
  elseif building_planet
    undo_set_project(planet)
    order.set_building_planet(selected_planet,building_planet)
  elseif activity

    undo_set_project(planet)
    order.set_activity(planet,activity)

  elseif building_improvement
    undo_set_project(planet)
    order.set_building_improvement(planet,building_improvement)
  elseif terraforming
    undo_set_project(planet)
    order.set_terraforming(planet,terraforming)
  
  end

end



local function default_candidate_click(el)

  if selected_planet.fixed_production
    return
  end

  if el.candidate.set_autobuild
    return set_auto(el)
  end

  call_with_flag('player_set_production',set_planet_project,el)

end

-- tooltip_grid_id:
-- little bit of tricky -- we have a number of
-- ui hover states that need UIDs.  hover, the candidate list
-- is regenerated after every order, which means most of the
-- natural sources of UIDs will change after each other,
-- leading to overlay flickering whenever we click on an
-- on-hover action.
--
-- however, it's easy enough to assign a grid location to each
-- hover state -- and those are unlikely to change much over
-- time.  
--
-- what's interesting about this hack is that it's actually
-- guaranteed that the same tooltip_grid_id will frequently be
-- used by different candidate elements.
--
-- however -- that doesn't actually end up mattering. 
-- the grid id is only used to determine alpha states,
-- and because those elements fade in and fade out quickly, the
-- worst that can happen here is for a quick screen flip to
-- cause an element in the same position as a different element
-- that was recently moused-over to show a very faint,
-- momentary, on-hover tooltip of it's own.
--
-- i've never been able to trigger this "bug" myself, even
-- though i know that, in theory, it should be possible to do
-- so.  as we expand the hotkey keymapping, it may become
-- easier to make it happen, but, even then, it's not a
-- behavior that really worries me.  worst case, i could just
-- add a third param for a string identifying the currently
-- selected planet, and/or the choose candidate mode, and that
-- would fix the issue, at the cost of a very small memory hit.
--
tooltip_grid_id = strong_memoizeNN | function(cx,cy)
  return {}
end
add_resize_callback | function()
  -- endless resize events could lead a 'memory leak' with this
  -- UID cache -- so we clear it in the even of a screen resize.
  tooltip_grid_id = strong_memoizeNN | function(cx,cy)
    return {}
  end
end


local draw_bk = restore_color | function (candidate)

  local ActivityArt, box_grow,element,set_frame_color4,call_in_color_scope,planet_pane in _ENV
  local ImprovementDrawers,print,t2s,drawers,box_grow,tooltip_grid_id in _ENV
  local planet=selected_planet
  local _ENV = candidate
  local el 

  if invalid
    set_frame_color4(.5,.5,.5,.9)
  end

  if drawfun 
    local bk_draw = frame.solid_candidate_bk

    --kludge, should be removed when we have mine/farm/market graphics
    if ImprovementDrawers.OutlineBK[candidate.building_improvement]
      bk_draw=frame.candidate_bk
    end 
    
    if outlined_bk
      bk_draw = frame.candidate_bk
    end
       
    bk_draw ( x,y,w,h )

    el = element | drawfun(x,y,w,'rounded')
  elseif special_item
    local bk_draw = frame.solid_candidate_bk
    bk_draw ( x,y,w,h )

  else
    el = element | frame.candidate_bk(x,y,w,h)
  end

  if name
    planet_pane.choose_title(name,frame.subtitle_hcenter(x,y))
  elseif bigtext
    frame.bigtext(x,y,bigtext)
  end

  if building_ground_unit
    local shrink=15
    drawers.PopIcons.normal_draw( get_ground_unit_icon(building_ground_unit),|) box_grow(-shrink,-shrink,x,y,w,h)
  elseif activity

    if invalid
      set_frame_color4(1,1,1,.25)
    end

    ActivityArt?[activity]?.draw?(x,y,w,h)
  elseif special_item 
     --  bk_draw=frame.candidate_bk
     --   bk_draw ( x,y,w,h )

    ActivityArt?[special_item]?.draw_rounded?(x,y,w,h)

    el = element(x,y,w,h)
  end

  --local el = external_frame_color(x,y,w,h,fr,ship_bk_color)
  el.on_hover=invalid and grey_cursor or hand_cursor
  el.candidate=candidate
  el.on_click= on_click or default_candidate_click
  el.on_rclick = candidate.on_queue_click or planet_pane.enqueue_click
  el.desc_text=desc_text
  el.mouseover_desc=true
  el.hover_id=tooltip_grid_id(candidate.x,candidate.y)
end

local function on_design_click(el)
  redesign_screen(el!.building_design)
end



local grey_icon = catch(.5,.5,.5,1)

local function tooltip_button(candidate,icon,cx,cy,text,invalid)
  local x, y in candidate  
  local icon_radius_hint = 17

  local sub_tooltip_id = tooltip_grid_id(cx,cy)

  local el = element | SCOPE_call(..icon_radius_hint,
    'icon_color',invalid and grey_icon,
    'edge_color_key',invalid and 'grey_edge',
    icon.draw, cx,cy)  

  --SCOPE_call(
  --    'word_color_key',invalid and 'invalid',
       draw_tooltip_cr( sub_tooltip_id,cx-icon_radius_hint-8,cy,text)

  el.tooltip_id=sub_tooltip_id

  if invalid
    el.on_hover=grey_cursor
  end

  return el
end

local function on_instant_refit_click(el)
  local candidate in el
  execute_order('instant_refit',|) function()
    refit_ship_to_option(candidate.ship,candidate.refit_option)
  end
end


local function draw_candidate_hover(candidate)
  
  local x,y,w,name,invalid,costs,ship,building_design,activity in candidate  
  local text_context in planet_bar

  local x1=x+8
  local y1=y+w-20
  

  y1-=34

  local ysep=40

  set_frame_color4(1,1,1,1)

  hover_id_to_speed[candidate]=4
  local planet = selected_planet

  --local ysep=22
  x1+=16

  x1=x+w-25
  y1=y+w-25

  if candidate.no_queue_button

  elseif candidate.refit_queue
    if planet.queue_full
      invalid=invalid or 'Queue Full'
    end

    if planet_subpane=='Refit'

      local el = tooltip_button(candidate,notification_icons.misc.queue,x1,y1,
                      id_to_message(invalid) or 'Queue Refit', invalid)

      if not invalid
        el.on_click = candidate.on_queue_click
        el.candidate = candidate
        el.add_to_queue=true
      end


      y1-=ysep

    end

    if not invalid
      local refit_option, instant_refit_text,instant_refit_invalid in candidate
      local coin_cost,metal cost in refit_option?
           
        

      local el = tooltip_button(candidate,notification_icons.misc.refit_for_coin,x1,y1,
                     instant_refit_text , instant_refit_invalid)       
           
      if not instant_refit_invalid
        el.on_click = on_instant_refit_click
        el.candidate = candidate
      else
        el.sound='INVALID_ACTION'
      end

      local mass_refit_info in candidate

      y1-=ysep

      if mass_refit_info

        local el = tooltip_button(candidate,notification_icons.misc.fleet_white,x1,y1,
                       mass_refit_info.text , mass_refit_info.invalid)   

        if mass_refit_info.invalid
          el.sound='INVALID_ACTION'
        else
          el.on_click in mass_refit_info
        end

      end

    end


  else

    local invalid = planet:cant_queue(candidate)

    local el = tooltip_button(candidate,notification_icons.misc.queue,x1,y1,
                    id_to_message(invalid) or 'Queue Production', invalid)

    if not invalid
      el.on_click = candidate.on_queue_click or planet_pane.enqueue_click
      el.candidate=candidate
    end


    y1-=ysep
  end

  if building_design

    local el = tooltip_button(candidate,notification_icons.misc.refit,x1,y1,'Edit Design')

    el.on_click = planet_pane.enqueue_click
    el.candidate=candidate
    el.building_design=building_design
    el.on_click = on_design_click

  end


end


local draw_item_overlay = restore_color | function (candidate)
  local x,y,w,name,invalid,costs,ship,building_design,terraforming,building_colony,building_planet in candidate

  local x1=x+8
  local y1=y+w-20
  

  if name
    if invalid
      set_frame_color4(.6,.6,.6,.9)

      --if ship
      --	frame.bigtext(x,y,"AT MAX")
      --end

    end
  end



  local text_context in planet_bar
  local ysep=18

  if costs

    local hammer_cost, metal_cost = costs()

    local str = tostring(hammer_cost)
    local word = text_context.lazy_word_render('item_overlay_font',  str..hammer_symbol())
    word.draw(x1,y1)

    --print(..metal_cost)

    if (metal_cost or 0) > 0
      y1-=ysep
      str = tostring(metal_cost)
      local word = text_context.lazy_word_render('item_overlay_font',  
        str..metal_symbol())
      word.draw(x1,y1)
    end
  
  end

  if terraforming or building_colony or building_planet

    local icon = (terraforming and 'terraform') or 
      (building_colony and 'colonization_flag') or
      (building_planet and 'artificial_planet' ) 

    if icon
--      SC
      SCOPE_frame_activation(
        frame_activation()*(invalid and .6 or 1),
        SCOPE_call,
          icon_color = invalid and grey or white,
          icon_radius_hint=24,shadow_opacity=.8,shadow_spread=.67,
          notification_icons.misc[icon].draw_shadowed_icon, x+w*.5, y+w*.5
      )
    end

  end

  y1-=ysep

--  local rx,ry, rw, rh = hammers_string(13,x1,y1,str)

  local hover_id = tooltip_grid_id(x,y)


  local hover_alpha = frame_activation() * (hover_alpha_states.state[hover_id] or 0)


 -- if candidate.activity=='Research'
 --   print(..hover_alpha)
 -- end

  if hover_alpha > 0
    -- print(..hover_id)
    SCOPE_frame_activation(hover_alpha,SCOPE_call,
      element_activation_thresh=.3,
      ..hover_id,
      draw_candidate_hover,candidate)
  end

end

local function request_planet(candidate)
  local x,y,w,invalid,building_colony in candidate
  local cx,cy=x,y
  cx,cy+=.5*w
  
  if candidate.building_planet 
    building_colony=building_planet_request_stub()
  elseif candidate.terraforming
     building_colony=terraforming_planet_request_stub(candidate.terraforming,selected_planet.size)   
  end

  local s = w*building_colony.drawer_scale


  local transform = new_request_transform( planet_inset_transform(cx-s*.5,cy-s*.5,s) )
  drawers[building_colony.drawer].request_planet_draws(transform,building_colony)
end


local function request_invalid_ships(candidate)
  local x,y,w,ship,invalid in candidate

  if candidate.building_planet
    if invalid return request_planet(candidate) end
  end

  if (not invalid) or not ship
    return
  end

  local drawer_table = drawers[ship.drawer]
  local transform = new_request_transform( drawer_table.rect_fit(ship,x,y,w,w) )
  drawer_table.request_ship_draws(transform, ship)

end


local function show_invalid_messages(candidate)

  local x,y,w,ship,invalid in candidate
  if (not invalid)
    return
  end

  local font,fh in fonts.choose_box!

  if invalid == 'orbitals_full'
    centered_text(font,fh,x+w*.5,y+w*.13,'Insufficient')
    centered_text(font,fh,x+w*.5,y+w*.13-fh*1.2,'Orbital Slots')
    return
  end

  if invalid == 'planet_full'
    centered_text(font,fh,x+w*.5,y+w*.13,'Maximum')
    centered_text(font,fh,x+w*.5,y+w*.13-fh*1.2,'Population Reached')
    return
  end

  if invalid == 'construction_tier'
    centered_text(font,fh,x+w*.5,y+w*.13,'Requires Additional')
    centered_text(font,fh,x+w*.5,y+w*.13-fh*1.2,'Orbital Infrastructure')
    return
  end

  if invalid == 'hull_locked'
    centered_text(font,fh,x+w*.5,y+w*.13-fh*.9,'Alien Design')
    return
  end


  --set_frame_color4(1,1,1,1)

  --[[
  if invalid=='need_metal'
    local building_design in candidate!
    local switch_metal = selected_planet : metal_to_switch(building_design)
    local text = shorthand_format | format ('Requires +%d<METAL>',switch_metal-gui_player().metal)
    planet_bar.invalid_project_text(x,y,w,text)
--    hcentered_text(font,fh,x+w*.5,y+w*.05,format ('Requires %d<METAL>',switch_metal))
  else
  --]]

  if invalid==true
  print(v2s | candidate)
  end

  hcentered_text(font,fh,x+w*.5,y+w*.05,id_to_message | invalid)
  

  --print(..invalid)
end

local draw_progress_bar do

  local color_set =  { 
    oranges = color_table {
      progress = 'f8971d',
      next_progress = '8f5909',
      empty = '28231c',
    },

    reds = color_table {
      progress = 'd43323',
      next_progress = '741c10',
      empty = '1f1c17',
    },

    greys = color_table {
      progress = catch(.7,.7,.7,.4),
      next_progress = catch(.2,.2,.2,.4),
      empty = catch(0,0,0,.4),
    },
  }



  local backing = fade_color_param('040912',59)

  draw_progress_bar = function (candidate)
    local progress_bar in candidate
    if not progress_bar
      return
    end

    local colors=color_set![progress_bar.colors]

    local x,y,w in candidate

 

    local bh=9
    local bw=round | w*.79
    local dx = round | .5*(w-bw)
    local dy = round | w*.12

    local bar0,bar1 in progress_bar

    --[[
      set_frame_color4 | backing()
      glBox | box_grow( 5,12, x+dx,y+dy+6,bw,bh)
      set_frame_color4 (1,1,1,1)

      local word = tooltip_text_context.lazy_word_render('vera12', 
        'Current Progress')
      set_frame_color4 (1,1,1,1)
      word.draw(x+dx,y+dy+bh+5)
    end
    --]]

    --local el = element()

    if candidate.invalid
      colors=color_set.greys
    end

    set_frame_color4 | colors.empty()
    glBox(x+dx,y+dy,bw,bh)

    if bar1
      set_frame_color4 | colors.next_progress()
      glBox(x+dx,y+dy,round | bar1 * bw,bh)
    end

    if bar0>0
      set_frame_color4 | colors.progress()
      glBox(x+dx,y+dy,round | bar0 * bw,bh)
    end


  end

end

local function request_ship(candidate)
  local x,y,w,ship,invalid,building_colony in candidate

  if building_colony
    return request_planet(candidate)
  end

  if candidate.building_planet
    if not invalid return request_planet(candidate) end
  end

  if candidate.terraforming
    if not invalid return request_planet(candidate) end
  end


  if invalid or not ship
    return
  end
  local drawer_table = drawers[ship.drawer]
  local transform = new_request_transform( drawer_table.rect_fit(ship,x,y,w,w) )
  drawer_table.request_ship_draws(transform, ship)
end

choose_hover_text = make_hover_text_cache { 
  name = 'choose_hover_text',
  text_width = 200,
  bk_color = catch(0,0,0,.75),
}

choose_hover_text.draw_for_candidate = restore_color | function(candidate)

  local x,y,w,h,building_improvement,invalid,ship,hover_text in candidate

  if candidate.name=='Artificial Planet'
  --  print(..hover_text)
  end

  local hover_id = candidate.hover_id or tooltip_grid_id(x,y)

  if (hover_alpha_states.state[hover_id] or 0)==0
    return
  end


  local cx,cy=x+w*.15,y+h*.35

  if hover_text
      choose_hover_text.hover_text(hover_id,cx,cy,hover_text)
  end


end

--[=[
local draw_hover_text = restore_color | function(candidate)

choose_hover_text.draw_for_candidate


--  choose_hover_text.hover_text(hover_id,cx,cy,'testing hover text: '..frame_time())
--  lazy_hover_text(hover_id,cx,cy,'testing hover text2: '..frame_time())
--  lazy_hover_text(hover_id,cx,cy,'testing hover text3: '..frame_time())


--[[
  local text = 

  if text
    choose_hover_text.hover_text(hover_id,cx,cy,text)
  elseif ship
    text = choose_ship_hover_text(ship) 
    SCOPE_call(,choose_hover_text.hover_text,hover_id,cx,cy,text)
  end
  --]]
end
--]=]

function redesign_screen(building_design)
	local src = gui_player().designs![building_design]
	local carrying_design
	if src.carried_design
		carrying_design = src
		src=src.carried_design
	end

  open_designer { 
    mode='Edit', 
    source_design = src,
    design_key = src._ikey,
  }
  designer_pane.call_on_close = nil
end

order_callback 'choose planet production' | function()
  draw_candidates=nil
end

local auto_sel1= { text_font = "fonts/bitstream/verabd.TTF", fh=23 }
local auto_sel0= { text_font = "fonts/bitstream/vera.TTF", fh=23 }

local set_zoom = conditional_zoom_handler | function()
  return (midscreen_pane == planet_pane) and planet_subpane
end

local vbox_list, scroll_position
local choose_frame_rect

function planet_pane.close_choose()
  planet_subpane=nil
  fallback_click_sound 'PANE_CLOSE'
  vbox_list.scroll_area.reset_scroll()

end

add_resize_callback | function()
  draw_candidates=nil

  local x,y,w,h=frame.inset()

  local rx,ry,rw,rh=frame.rect()
  ry-=13
  rh+=58

  choose_frame_rect= catch(rx,ry,rw,rh)

  vbox_list = framed_scrolling_vboxes {
      --..text_context,
      xpad_text=20,
      suppress_resize_warning=true,
      text_context='none',
      rect = catch | ChooseFrame.clipping_rect(rx,ry,rw,rh),
    }


  scroll_position = ChooseFrame.make_scroll_position(
      vbox_list.scroll_area,
      rx,ry,rw,rh,
    )

  local handle_zoom_in_rect in vbox_list!.scroll_area

  set_zoom | function(dz)
    return  vbox_list.scroll_area.handle_zoom_in_rect(dz,
      rx,ry,rw,rh
      ) 
  end

end



function draw_choose()

  if not draw_candidates

  --[[
   local text_context = new_text_rendering_context { 
      name = 'ChooseProductionText',
      suppress_resize_warning=true,
    }
    --]]

    --console(..candidates_h)
    --print(..vbox_list.scroll_area.scroll)

    local scroll = vbox_list.scroll_area.scroll
    vbox_list.clear()
    vbox_list.scroll_area.scroll=scroll

    --vbox_list.scroll_area.reset_scroll()


    local production_candidates, refit_candidates in calc_production_candidates(planet_pane.limit_production_candidates)
    local candidates = production_candidates


    if (planet_subpane=='Refit' ) or (planet_subpane=='Refit Queue')
      if refit_candidates
        candidates = refit_candidates
      else
        -- this is a rare case, but, it can happen if, say, we hit
        -- end-turn while in the refit pane.
        planet_subpane='choose_production'
      end
    end


    local candidates_h = frame.arrange_candidates( candidates )

    local function call_for_candidates(f)
      for i,candidate in ipairs(candidates) do
        f(candidate)
      end
    end




    --local x,y,w,h= ChooseFrame.clipping_rect(rx,ry,rw,rh)

    local draw =  safe_requests | restore_color | function ()
      

      call_for_candidates(draw_bk)
      call_for_candidates(request_ship)
      execute_draw_requests()
      set_frame_color4(1,1,1,1)
      call_for_candidates(request_invalid_ships)
      set_frame_color4(1,1,1,.25)
      execute_draw_requests()

      set_frame_color4(1,1,1,1)
      call_for_candidates(draw_progress_bar)

      set_frame_color4(.6,.6,.6,.8)
 --     set_frame_color4(.8,.3,.2,1)
      call_for_candidates(show_invalid_messages)
      set_frame_color4(1,1,1,1)
      call_for_candidates | draw_item_overlay

      
    end

    local function title()
      if (planet_subpane=='Refit' ) or (planet_subpane=='Refit Queue')
        return  "Choose Ship for Refit:"
      else
        return "Choose Production for " .. selected_planet.name .. ":"
      end
    end

    vbox_list.add_vspace(4)

    vbox_list.add_vbox(function()
      display_text( fonts.bold, 18, 32,0, title():upper() )
    end,16)


    vbox_list.add_vspace(16)

    vbox_list.add_dynamic_vbox(draw,
      catch | candidates_h
    )

    --[[
    local scroll_area = frame.make_scroll_area(_ELEMENT_ENV) {
      draw_height = catch | candidates_h,
      push_zoom_handler = set_zoom,
      ..draw,
    }
    --]]

    vbox_list.scroll_area.speed_mult=6


    --print(..vbox_list.scroll_area.scroll)


    function draw_candidates()

      --frame.draw_bk()
      ChooseFrame.draw_bk | choose_frame_rect()

      --do return end

      --scroll_area.draw()
      --frame.draw()

      --glBox | ChooseFrame.bk_rect(rx,ry,rw,rh)



      vbox_list.clipped_draw( )

      ChooseFrame.draw_frame | choose_frame_rect()
      ChooseFrame.draw_top_buttons | choose_frame_rect()

      --ChooseFrame.draw_title(title(),rx,ry,rw,rh)

      scroll_position.draw()

      --frame.gen_title( title() )
      --frame.small_title(title())

      --frame.draw_scroll()
    end

    local function draw_hover_items()
      call_for_candidates | choose_hover_text.draw_for_candidate
    end

    function draw_candidates_hover()

      vbox_list.unclipped_drawfun(draw_hover_items)

    end



    optional_gc()
  end

  draw_candidates()

  insert(  ChooseFrame.close_rect_element( choose_frame_rect ),
    on_click= planet_pane.close_choose,
    on_hover= grey_cursor,
    handle_in_transport_mode=true,
  )
  
end


local function buy_click(el)
  order.buy_production(selected_planet)
  Sounds.PURCHASE_ITEM()

--  if planet_subpane=="choose_production"
--    planet_subpane=nil
--  end
end

local function production_click(el)
  local planet = selected_planet
  if shift_down() 
    if planet.empire.designs[planet.building_design]
      redesign_screen(planet.building_design)
    end
  else
    if planet_subpane~='choose_production'
      fallback_click_sound 'PANE_OPEN'
    end
    planet_subpane= "choose_production"
    clear_layout()
  end
end


draw_production =  restore_color | function (planet)
  local hide_queue
  local building_ground_unit,building_planet,
    building_colony,empire,building_improvement,building_design,activity,terraforming,
    refitting,refitting_orbital in planet

  local _TRACE_INFO = planet.name

--	Design.check_empire(empire)




  local el = element( planet_bar.production_rect_bk() )

  if selected_planet.fixed_production
  --  el.on_hover = grey_cursor
    el.on_click = production_click
  else
    el.on_click = production_click
--    if shift_down() el.on_hover=purple_cursor end
  end


  local hull_desc_key

  if building_improvement
    local x,y,w,h=planet_bar.production_full_rect()
    local s=max(w,h)

    local sx,sy=0,0

    if production=="Factory"
      sx=-s*.02
      s*=1.05
    end

    clipped_box(x,y,w,h,ImprovementDrawers![building_improvement],sx,sy, s)

  elseif (refitting or refitting_orbital)
    local ship = refitting or refitting_orbital
    local drawer_table = drawers[ship.drawer]
    local x,y,s,degree = drawer_table.rect_fit(ship, planet_bar.production_inset_rect())
    local transform = new_request_transform( x,y,s, degree )
    call_with_gl_white(drawer_table.request_ship_draws,transform, ship )
    execute_draw_requests()


  elseif building_colony or building_planet or terraforming
    local planet=
      building_colony
      or (terraforming and terraforming_planet_request_stub(terraforming,selected_planet.size)  ) 
      or building_planet_request_stub()
    


    local drawer_table = drawers[planet.drawer]
    local x,y,s,degree = drawer_table.rect_fit(planet, planet_bar.production_inset_rect())
    --glBox( x,y,s, s)
    local transform = new_request_transform( x,y,s,degree )
    drawer_table.request_planet_draws(transform,planet)
    execute_draw_requests()

    local icon = (terraforming and 'terraform') or 
      (building_colony and 'colonization_flag') or
      (building_planet and 'artificial_planet' ) 

    if icon

      SCOPE_call(icon_radius_hint=35,shadow_opacity=.8,shadow_spread=.67,
        notification_icons.misc[icon].draw_shadowed_icon, x+s*.5, y+s*.5)

    end

  elseif building_ground_unit
    local x,y,w,h= planet_bar.production_inset_rect()
    local shrink=h*.03
    if building_ground_unit:match 'raider'
      shrink=h*.1
      y-=h*.05
    end
    drawers.PopIcons.normal_draw(get_ground_unit_icon(building_ground_unit),|) box_grow(-shrink,-shrink,x,y+h*.02,w,h)
    
  elseif building_design
  
    local design = all_designs![building_design]
    assert(planet.empire==design.empire)
    hull_desc_key in design

    local ship = design_to_ship(design)
     
    local x,y,s,degree = drawers[design.ship_drawer].rect_fit(ship, planet_bar.production_inset_rect())

    --local x,y,s,s0,yshift,degree = calc_shifts(design.ship_drawer,x,y,s)
    local transform = new_request_transform( x,y,s, degree )
    
    call_with_gl_white(design.drawer_t.request_ship_draws,transform, ship )


    --design.request_parts(transform,empire)
    execute_draw_requests()
  elseif activity
   
    local rx,ry,rw,rh=planet_bar.production_full_rect()
    ry+=12
    ActivityArt[activity]?.draw?(rx,ry,rw,rh)

  else
    local text_context in planet_bar
    local word = text_context.lazy_word_render('orig36','Click to Select Production')

    local x,y,w,h = planet_bar.production_inset_rect()


    if not word.draw_shadow
      local shadow_options = {sigma=4,spread=.7,opacity=.4,dist=2}
      shadowed_word(shadow_options,word)
    end

    --word.draw_hcentered(x+w*.5,h+h*.5)
    local tx,ty=x+w*.5,y+h*.5
    tx-=.5*word.w
    ty-=word.h*.5
    word.draw_shadow(tx,ty)
    word.draw(tx,ty)
    hide_queue=true

    local icon_radius=18
    SCOPE_call(..icon_radius,
      shadow_spread=.65,
      notification_icons.misc.wrench_II_white.draw_heavy_shadowed,x+icon_radius+15,y+h-20-icon_radius)

--    glBox | planet_bar.production_inset_rect()
      --fconsole('%d',transform)
   end



  local p=planet

  local ptext = p.active_project_name
  local draw_automate_overlay
  if (ptext~='None') and (not p.player_set_production)
    ptext..="*"

    if  (not p.fixed_production)
      draw_automate_overlay=true

      local text_context in planet_bar
      local word = text_context.lazy_word_render('orig36','Production Automated')

      local x,y,w,h = planet_bar.production_inset_rect()


      if not word.draw_shadow
        local shadow_options = {sigma=4,spread=.7,opacity=.4,dist=2}
        shadowed_word(shadow_options,word)
      end


      local tx,ty=x+w*.5,y+10
      tx+=30
      tx-=.5*word.w
      word.draw_shadow(tx,ty)
      word.draw(tx,ty)
      --hide_queue=true

      ---[[
      local icon_radius=18
      SCOPE_call(..icon_radius,
        shadow_spread=.65,
        notification_icons.misc.automate.draw_heavy_shadowed,tx-icon_radius-15,ty+14)
        --]]

    end
  end

  local _TRACE_INFO = p.project_name

  if p.activity and p.activity~='Stockpile'

    planet_bar.big_production_title(ptext)

  else

    planet_bar.production_title_bk()

    local turns, hammer_cost = p:turns_to_complete()

    --print(..turns,..cost,..planet.project)

    insert(element | planet_bar.production_title(ptext),
      desc_text = hull_desc_key,
    )
    
    if(turns)

      if turns==max_double
        planet_bar.turns_left "??? turns left"
      elseif turns>1
        --print(..turns)
        planet_bar.turns_left("~%d turns left", turns)
      else
        planet_bar.turns_left "1 turn left" 
      end


      if planet_blockaded(planet)
        local el = element (planet_bar.blockaded_text() )
        el.on_hover=grey_cursor
      elseif planet.show_metal_warning
        planet_bar.metal_shortage_text() 

      end

      do

        local buyout,hammers,metal = p:buy_cost()

        if (buyout>0) or (metal>0)

          local can_buy = (not planet_blockaded(p)) and buyout <= p.empire.credits

          buy_string = format('%d<COIN>',buyout)
          local text_context in planet_bar

          if metal>0
            buy_string ..= format(', %d<METAL>',metal)
          end

          if can_buy 
            can_buy = empire.metal >= metal
          end

          local text="Buy: " .. shorthand_format| buy_string

          local el = element | planet_bar.buy_text(can_buy,text)		
         
          if can_buy
            el.on_hover=hand_cursor
            el.on_click = buy_click
          else
            el.on_hover=grey_cursor
          end
        end

      end

      local mult = planet.active_project_discount

      planet_bar.progress_text( format("%d/%d",planet.hammers+planet.overflow_hammers,hammer_cost),mult)

    else
      planet_bar.progress_text( format("%d/--",planet.hammers+planet.overflow_hammers))
    end

  end

  if selected_planet.fixed_production
    planet_bar.cannot_change_warning()
  end


  return hide_queue
end



end
