UI_File | function(_ENV)

depends 'pack_ships'

order_callback 'FleetShipSelection' | function()
  if get_selected_fleet()
    gui.GalaxyMap.reset_motion_selections()
  end
	if FleetMove==star_selection_mode
		star_selection_mode=nil
	end
end

local fleet_panel = myskin.FleetPanel(_ENV)

insert(_ENV,..fleet_panel)

local checked_drawer_cache
fleet_pos_cache = {}

function reset_ship_layouts()
  fleet_pos_cache = {}
  checked_drawer_cache=false
end

local selected_for_motion = {}
local show_selection_glows

function reset_motion_selections()
  cleartable(selected_for_motion)
  show_selection_glows=nil

  for ship ; get_selected_fleet()?.ships?
    selected_for_motion[ship]=not ship.inactive
  end
end

function ships_selected_for_motion()
  return selected_for_motion
end

local fleet_glows = MarkedAlphaState1(2*math.pi,|) {}

local fleet_neutral_glows = MarkedAlphaState1(2*math.pi,|) {}

function on_key_selected.clear_motion_selection(selected_map_key,old_key)
  if selected_map_key~=old_key
    reset_motion_selections()
  end
end

function get_motion_selection()
  return get_selected_fleet() and selected_for_motion
end

local function cant_move_fleet(fleet)
  if not can_command_fleet(fleet)
    return 'Fleet outside of command range'
  end

  if fleet.civilians
    return 'Ships under civilian command'
  end




end


function inhibited_fleet_move(fleet)

  local cant = cant_move_fleet(fleet)
  if  cant
    return cant
  end

  local star = at_star(fleet)
  if star
      for i, report ; fleet.empire.reports
        if (report.star==star) 
        
          if (report.type == 'system_encounter') and (report.star==star) and not report.resolved
 

            return  'Encounter at '..star.name

         
          end

          cant = inhibit_fleet_motion(report)
          if cant
            return cant 

 
          end

        end

      end
    end

end


function on_hover_update.fleet_glows()
  fleet_glows.update()
  fleet_glows.clear_marks()

  fleet_neutral_glows.update()
  fleet_neutral_glows.clear_marks()

end

function transport_assign_click(el)
  local ship,fleet in el!
  order.assign_ship_to_trade(ship,nil,true)
  if _undefined(fleet)
    select_map_key | nil
  end
  --select_map_key | fleet_to_id | fleet
end

function ShipProperty.tooltip_name(ship)
  local rval=ship.name
  if ship.mercenary_empire
    rval= 'Mercenary '..ship.name
  end

  local displayed_hp_percent,inactive in ship

  if inactive
    rval..='\nInactive'
    if inactive=='refitting'
      local refit_progress in ship!
      rval..=format('\nRefit Progress: %d%%',round | refit_progress*100)
    end
  elseif displayed_hp_percent
    rval..=format('\nHull Integrity: %d%%',round | displayed_hp_percent * 100 )
  end

  return rval
end



local display_damage_bars_as_needed do
  local color_set =  { 
    oranges = color_table {
      progress = 'f8971d',
      next_progress = '8f5909',
      empty = '28231c',
    },

    reds = color_table {
      progress = 'cf111a',
      next_progress = '741c10',
      empty = '521b1e',
    },
  }


  local draw_bars = function(damage_bar, colors, bar0,bar1)
    local c = color_set[colors]
    local x,y,w,h=damage_bar()
    set_frame_color4 | c.empty()
    glBox|round_rect(x,y,w,h)

    if bar1
      set_frame_color4 | c.next_progress()
      glBox|round_rect(x,y,w*bar1,h)
    end

    if bar0>0
      set_frame_color4 | c.progress()
      glBox|round_rect(x,y,w*bar0,h)
    end

  end

  display_damage_bars_as_needed = restore_color | function (packed,fleet)
    for i,pos in ipairs(packed.ship_positions)

      local ship in pos
      local hp_percent = ship.displayed_hp_percent
      local refit_progress in ship

      if refit_progress
        draw_bars(pos.damage_bar,'oranges',ship:expected_refit_progress())
      elseif hp_percent
        draw_bars(pos.damage_bar,'reds',ship:expected_hp_percent())
      end
    end
  end

end

local add_damage_bar_positions =  function(packed)
  --local iw,ih, scale,ship_dims,dx in packed.pack_result
  local suggest_row_shifts in packed  


  for i,pos in ipairs | packed.ship_positions

  --[[
      local ship,unscaled in pos 

      local res=15
      local sub=3

      local bitmap,bits = ship_alpha_bitmap(ship,sub*res*unscaled.w,sub*res*unscaled.h, 
        ship.drawer_table.fleet_scale or 1,res)

      local x0,y0 = unscaled.x,unscaled.y-(suggest_row_shifts?[unscaled.x] or 0)
    
      for x = 0,(unscaled.w*sub -1)for y=0, (unscaled.h*sub-1)
        if bits:get(x,y)
          print(..x,..y)
        end
      end end

      --]]

--      print(v2s | pos)
      local ship,unscaled in pos

      local x,y,s = packed_draw_position(pos)
      --print(..x,..y,..s)

      --print( drawer_offset_info 

      local layer_info in ship.drawer_table!
      local x0,y0,w,h in layer_info.sub_canvas!


      local canvas_size = layer_info!.canvas_size

      local m = s / canvas_size

      local max_w = unscaled.w*packed.pack_result.scale

      local bw = min( 58, round | max_w)
      local bh = 4


      pos.damage_bar = catch (
        x+ max( m*x0, .5*(s-bw)-23 ),
        y+ y0*m,
        bw,
        bh
      )

  end

end


--
-- this icon_flex stuff is the result of an afternoon when i
-- was feeling kinda crazy.  the basic idea is to:
--
-- 1) get a low-res bitmap representing the implied area
-- covered by each ship in the fleet.
--
-- 2) approximate the distance between the position where the
-- fleet empire icon will be displayed and the closest occluded
-- bit in the low-res bitmap.
--
-- 3) use that distance approximation to determine how closely
-- we can nudge the insignia icon to the displayed fleet. 
-- ships that fill up almost their entire allocated display
-- area will prevent us from having any effective 'icon_flex',
-- but, in cases where we have ships with a pointed bow, the
-- icon will be able to slide to the left by 20-40 px.  it's a
-- subtle adjustment, for all the complexity that goes into
-- coding it up.  also: the implementation is fairly
-- inefficient, because i'm already putting way more time into
-- this feature than it deserves.
--
local icon_flex = weak_memoizeNN | function(packed,fleet)
  local iw,ih, scale,ship_dims,dx in packed.pack_result
  local x,y,h,w,dx2,suggest_row_shifts in packed

  local cx,cy,icon_radius_hint do
    icon_radius_hint=60*.5 / scale

    local icon = fleet.empire.insignia_icon
    local x,y,w,h=bottom_bar.rect()
    h/=scale
    
    cx= - packed.insignia_xpad*.5/scale

    cy = h-icon_radius_hint
  end


  local min_dist

  for i,pos in ipairs | packed.ship_positions
    local ship,unscaled in pos 

    local res=15
    local sub=3

    if unscaled.x>0 goto next_ship end

    local bitmap,bits = ship_alpha_bitmap(ship,sub*res*unscaled.w,sub*res*unscaled.h, 
      ship.drawer_table.fleet_scale or 1,res)
    set_frame_color4(1,0,1,.3)
    
    local x0,y0 = unscaled.x,unscaled.y-(suggest_row_shifts?[unscaled.x] or 0)
    
    local bx,by=unscaled.x,unscaled.y-(suggest_row_shifts?[unscaled.x] or 0)


    for x = 0,(unscaled.w*sub -1)for y=0, (unscaled.h*sub-1)
      
      if bits:get(x,y)
        local x0,y0=bx+x/sub,by+y/sub
        local s=1/sub

        min_dist = generic_min(
          
          point_dist(cx,cy,x0,y0+.5*s)-icon_radius_hint,

          min_dist
        )

      end

    end end

    ::next_ship::

  end

  if min_dist
    min_dist*=scale
  end

  return min_dist or 0
end

local function pack_fleet(fleet)
  local layout_w,layout_h in layout

  local r=fleet_pos_cache[fleet]
  if(r) return r end

  local right_align
  
  local x,y,w,h=bottom_bar.rect()

  local move_button = fleet.empire==gui_player() 
  
  --local move_w in fleet_panel

  local move_w=120
  local move_xpad=10
  local min_x=120
  
  local insignia_xpad=40


  w-=2*insignia_xpad
  

  if move_button
    w-=move_w+2*move_xpad+min_x
  end

  --print(t2s | fleet.ships)

  local r = pack_ships(fleet.ships,..x,..y,..w,..h,..right_align)
  local w=r.w

  local dx=move_button and max(move_w+min_x,.5*(layout_w-w)) or .5*(layout_w-w)

  insert(r,dx2=dx,..insignia_xpad,..move_xpad)

  --dx+=insignia_xpad

  local icon_flex = icon_flex(r,fleet)
  
  --print(..icon_flex)
  icon_flex=min(60,max(0,icon_flex-6))

  insert(r,..icon_flex)

  if move_button
    r.move_x = r.x+dx-move_w-move_xpad -6
    r.move_w=move_w

    r.move_x-=insignia_xpad-icon_flex
    --r.insignia_xpad-=icon_flex
  end




  -- center the result 
  for _,p ; r.ship_positions
    if right_align
      p.x-=.5*(screen_w-w)
    else
      p.x+=dx
    end

    local shifty = r.suggest_row_shifts?[p.unscaled.x] or 0
    --print(..shifty)
    p.y-=shifty*r.pack_result.scale

  end

  add_damage_bar_positions(r)

  fleet_pos_cache[fleet]=r

  return r
end


function ship_view(el)
  local ship = el.ship

  if midscreen_pane==ship_status_pane and ship_status_pane.object==ship
    close_midscreen_pane()
  else

    open_midscreen_pane(ship_status_pane,ship)
  end

end

FleetMove = FleetMove or {}



function FleetMove.invalid()
  local fleet in FleetMove

  if _undefined(fleet)
    return true
  end

  if fleet.empire~=gui_player() or not can_command_fleet(fleet)
    console("invalid star_selection_mode.fleet",fleet)
    return true
  end
end

function FleetMove.title(star)
  --local fleet in FleetMove?
  local fleet = get_selected_fleet()
  if not fleet return end

  local time,dist = fleet_travel_time(fleet,star) 
  local valid = fleet : can_reach (star.x,star.y,selected_for_motion!)
  local name = star.name

  if not gui_player().explored[star]
    name=uncharted_star_title![star.type]
  end

  if valid
    --bottom_bar.title("%s: %d turns", star.name, fleet_travel_time(moving_fleet,star) )
    return format("%s: %d turns (%.01fpc)%s", name, time, dist, valid=='some' and '  WARNING -- NOT ALL SHIPS HAVE SUFFICIENT RANGE' or '' )
  else
    return format("%s: insufficient range (%.01fpc)", name, dist )
  end    
end



function move_to_star_as_able(fleet,star)
  
  remove_missing_keys(selected_for_motion, fleet.ships)
   
  if not next | selected_for_motion
    Sounds.INVALID_ACTION()
    return
  end



  local valid = fleet : can_reach (star.x,star.y,selected_for_motion!)

  --print(valid, v2s | selected_for_motion)

  if valid=='all'
    Sounds.FLEET_MOVE_CONFIRMED()
    order.set_fleet_dest(fleet,star,selected_for_motion)
    reset_motion_selections()

   else
      Sounds.INVALID_ACTION()
     --error 'todo -- remove fleet splitting'
  end

end

function FleetMove.on_select_star(star)
  local fleet in FleetMove!

  move_to_star_as_able(fleet,star)

  select_map_key | fleet_to_id(fleet)
end

local function move_click(el)
  local fleet in el
  
  --if midscreen_pane==ship_status_pane
    close_midscreen_pane()
  --end


  if fleet.civilians
    error 'case no longer supported'
    --transport_src = fleet
    --transport_colonists = nil
    --fleet_transport=fleet
  else
    FleetMove.fleet=fleet
    star_selection_mode=FleetMove
  end

  Sounds.SELECT_FLEET()

end

local function invert_selections(el)
  
  Sounds.SELECT_SHIP_IN_FLEET()

  if not show_selection_glows
    show_selection_glows=true
    return
  end

  for ship ; el.fleet.ships
    if selected_for_motion[ship] or ship.inactive
      selected_for_motion[ship]=nil
    else
      selected_for_motion[ship]=true
    end
  end 

end


local function toggle_ship_selection(el)
  --console(el)

  if el.ship.inactive
    show_selection_glows=true
    Sounds.INVALID_ACTION()
    return
  end

  if not show_selection_glows
    cleartable(selected_for_motion)
    toggle(selected_for_motion,el!.ship)
    show_selection_glows=true
    Sounds.SELECT_SHIP_IN_FLEET()

    --Sounds.
    return
  end

  toggle(selected_for_motion,el!.ship)

  if selected_for_motion[el.ship]
    Sounds.SELECT_SHIP_IN_FLEET()
  else
    Sounds.DESELECT_SHIP_IN_FLEET_SOURCE()
  end

end

local draw_range_warning = restore_color | function(p)

  local ship,x,y,w,h in p

  local alpha = ship_range_warnings.state[ship] or 0

  if alpha == 0
    return
  end

  local cx=x+w*.5
  local cy=y+h*.5

  --set_frame_color4(0,0,0,alpha)
  --glBox(x,y,w,h)

  local text = get_tooltip('Insufficient Range!')

  text.draw_center(cx,cy)

end

local selected_ship_draw = restore_color | safe_requests | function(ship_positions, unselected,move_x,engines_on)

  glColor4d(1,1,1,1)
  local needs_draw
  for _,p ; ship_positions
    local ship in p

    glColor4d(1,1,1,1)

    local x,y,s = packed_draw_position(p)
    needs_draw=true
    local transform = request_recorded_ship_draw(ship, x,y,s, 0,nil,engines_on )
    --print(..transform)
		--glBox(x,y,s,s)

    if move_x 
    
      if show_selection_glows
        if selected_for_motion[ship]
          fleet_glows.mark(ship)
        end
      else
        --fleet_neutral_glows.mark(ship)
      end
    
      local a0=(fleet_glows.state[ship] or 0)
      if a0 > 0
        ship.drawer_table.request_halo(transform,ship,'fleet_selection', a0 )
      end

      local a1=(fleet_neutral_glows.state[ship] or 0)
      if a1 > 0
        ship.drawer_table.request_halo(transform,ship,'fleet_selection',.6*a1 )
      end



    end
  end

  set_frame_color4(1,1,1,1)
  execute_draw_requests()
end

local function in_box(i,j,info)
  return (i>=info.x) and (j>=info.y)
    and ( (info.x+info.w) > i) and ( (info.y+info.h) > j )
end

local draw_fleet_packing_grid = restore_color | gl_matrix_fun | function(packed,fleet)
  local iw,ih, scale,ship_dims,dx in packed.pack_result
  local x,y,h,w,dx2,suggest_row_shifts in packed

  
  glTranslate2D(dx2,0)

  glScale2D(scale,scale)

  local nships = #packed.ship_positions
  local highlight= packed.ship_positions[ (floor(1.5*frame_time())%nships) + 1 ]

  for i = 0, iw-1
    if i*scale < w
      local shifty = suggest_row_shifts?[i] or 0

      for j = 0, ih-1
    
        if in_box(i,j,highlight.unscaled)
          set_frame_color4(.5,.5,1,.5) 
        else
          set_frame_color4(1,1,1,.2)
        end

        glBox | box_grow(-.05,-.05,i,j-shifty,1,1)

      end
    end
  end




--  memconsole(v2s | packed)
--  print(..x,..y,..iw,..ih,..dx,packed.x0)
end


local draw_fleet_occlusion_grid = restore_color | gl_matrix_fun | function(packed,fleet)
  local iw,ih, scale,ship_dims,dx in packed.pack_result
  local x,y,h,w,dx2,suggest_row_shifts in packed



  
  glTranslate2D(dx2,0)
  glScale2D(scale,scale)


  local cx,cy,icon_radius_hint do
    icon_radius_hint=60*.5 / scale

    local icon = fleet.empire.insignia_icon
    local x,y,w,h=bottom_bar.rect()
    h/=scale
    
    cx= - packed.insignia_xpad*.5/scale

    cy = h-icon_radius_hint
    --[[
    SCOPE_call(..icon_radius_hint,
      icon.simple_draw,
    cx, cy )
    --]]
  end

  local min_dist

  for i,pos in ipairs | packed.ship_positions
    local ship,unscaled in pos 

    --local s = max(unscaled.w,unscaled.h)*(or 1)
    local res=15
    local sub=3
    --local xshift=.5*(unscaled.w-s)
    --local yshift=.5*(unscaled.h-s) - (suggest_row_shifts?[unscaled.x] or 0)
    local bitmap,bits = ship_alpha_bitmap(ship,sub*res*unscaled.w,sub*res*unscaled.h, 
      ship.drawer_table.fleet_scale or 1,res)
    set_frame_color4(1,0,1,.3)
    
    local x0,y0 = unscaled.x,unscaled.y-(suggest_row_shifts?[unscaled.x] or 0)
    
    --bitmap:draw_in_box(x0,y0,unscaled.w,unscaled.h)
    local bx,by=unscaled.x,unscaled.y-(suggest_row_shifts?[unscaled.x] or 0)



    --bits:reduce(res)

    for x = 0,(unscaled.w*sub -1)for y=0, (unscaled.h*sub-1)
      --print(..x,..y)
      
      if bits:get(x,y)
        local x0,y0=bx+x/sub,by+y/sub
        local s=1/sub
        
        glBox | box_grow(-.05/sub,-.05/sub,x0,y0,s,s)

        min_dist = generic_min(
          
          point_dist(cx,cy,x0,y0+.5*s)-icon_radius_hint,

          min_dist
        )

      end

    end end

  end

  if min_dist
    min_dist*=scale
  end

  memconsole(..min_dist)

end

local icon_tooltip_el, get_icon_element

add_resize_callback | function()

  icon_tooltip_el = new_tip_maker { 
    delayed_draw_context = dropdown_menu!,
    style = 'draw_center_left', 
    dx=60,dy=30, }

  get_icon_element = strong_memoizeNN | function(text)
    return icon_tooltip_el(text)
  end

end
--jem.scrap.fleet
local function scrap_fleet(el)
  execute_order('scrap_fleet',|) function()
    for scrap_ship ; el.fleet.ships
      el.fleet:scrap(scrap_ship)
    end
  end
  Sounds.SCRAP_SCUTTLE_SHIP()
end
--jem.scrap.fleet.end
local draw_fleet_ships  = restore_color | function(fleet)

  fleet = id_to_view_fleet | fleet._ikey

  if not fleet return end

  local player=gui_player()
  local revealed = revealed_stars()

  local packed = pack_fleet | fleet

  local ship_positions,move_x,move_w,pack_result,icon_flex in packed

  bottom_frame.draw_bk()

  local id=fleet_to_id(fleet)

  local ships=fleet.ships

  pal=fleet.empire.tactics.palette

  if options.display_fleet_packing_grid
    draw_fleet_packing_grid(packed,fleet)
  end



  local can_command = can_command_fleet(fleet) and fleet.empire==player and (not fleet.civilians )
  SCOPE_call(..fleet, selected_ship_draw, ship_positions,get_motion_selection(),move_x and can_command,fleet.dest_star)

  
  local element in _ENV

 
  display_damage_bars_as_needed(packed,fleet)

  for _,p in ipairs | ship_positions
    local x,y,w,h,ship in p

    local el=element(x,y,w,h)
      
    if move_x
      el.on_click= move_x and toggle_ship_selection
      el.on_hover=green_cursor
      el.on_rclick=invert_selections

      draw_range_warning(p)

    end

    

    el.ship=ship
    el.transport_key=ship
    el.fleet=fleet
    el.hover_id = ship
    el.tooltip_id=ship
    
    --glBox(x,y,w,h)
  end



   do
    local icon_radius_hint=60*.5
    local icon = fleet.empire.insignia_icon
    local x,y,w,h=bottom_bar.rect()
    --local cx=move_x+move_w+packed.move_xpad*.25+icon_radius_hint
    local cx=packed.x-packed.insignia_xpad*.5+packed.dx2+icon_flex
    local cy = h-icon_radius_hint
    
--    local el = element | centered_box(cx,cy,2*icon_radius_hint,2*icon_radius_hint)
    local el = get_icon_element(fleet.formal_name) | SCOPE_call(..icon_radius_hint,
      icon.simple_draw,
        cx, cy )
		el.desc_text=nil

		if gui_player() : can_contact(fleet.empire)
			el.empire=fleet.empire
	    el.on_click = function(el) 
				Sounds.SCREEN_OPEN()
				open_diplo_pane()

				diplo_pane.transaction = nil

				diplo_pane.discussion = {
					your_empire=gui_player(),
					their_empire=el.empire,
				}

				SCOPE_call(discussion=diplo_pane.discussion,
					diplo_pane.start_discussion, diplo_pane.discussion)

				diplo_pane.discussion.order_rebuilt=true

			end 
		else
			el.on_click =  function(el) 
				Sounds.INVALID_ACTION()
			end
		end

  end 

  for _,p in ipairs | ship_positions

    draw_info_icon(p,can_command,fleet)

  end


  --move_button=false

  if move_x    
    set_frame_color4(1,1,1,1)
    bottom_frame.draw_button_bk(move_x,move_w)
    local cant =  cant_move_fleet(fleet)
    local icon = notification_icons.misc[ cant and 'spaceship_grey' or 'spaceship_white']
    
    local cant_click
    local cant_report


    if not cant
      local star = at_star(fleet)
      for i, report ; fleet.empire.reports

        if (report.star==star) 
          if (report.type == 'system_encounter') and not report.resolved


            icon = newicon.system_encounter.get_icon(report)
   
            
            cant = 'Encounter at '..star.name
            cant_report= report
            break

          end

          cant = inhibit_fleet_motion(report)
          if cant
            cant_report = report
            icon = newicon[report.type]
            break
          end
        end

      end
    end

    local none_selected = no_ships_selected(fleet.ships,ships_selected_for_motion())

    if none_selected
      icon =notification_icons.misc.spaceship_grey
    end

    local cx,cy=move_x+move_w*.5,bottom_bar.height*.75

    local hover_id=fleet

    --print(..hover_id)

    local el = med_tooltip_icon(hover_id, icon,cx,cy)
--jem.scrap.fleet
    local scrap_hover_id='scrap_this_fleet'
    local scrap_icon = notification_icons.misc.recycle
    local scrap_el = med_tooltip_icon(scrap_hover_id, scrap_icon,cx,bottom_bar.height*.45)
    med_icon_tooltip(scrap_hover_id, cx, bottom_bar.height*.45, "Scrap Fleet")
    scrap_el.fleet=fleet
    scrap_el.on_click = scrap_fleet
--jem.scrap.fleet.end
    --local color = can_command_fleet(fleet) and blue or grey


    el.fleet=fleet
    el.on_hover=state=="invalid" and grey_cursor or purple_cursor

    --print(..el.hover_id)

    bottom_frame.draw_button_frame(move_x,move_w)
  
    if star_selection_mode?.fleet
      hover_alpha_states.mark(hover_id)
    end

    if not cant
      if none_selected
        med_icon_tooltip( hover_id, cx, cy, fleet.all_inactive and "Can't Move Inactive Ships" or "No Ships Selected")
        el.on_hover=grey_cursor
        el.sound='INVALID_ACTION'
      else
        med_icon_tooltip( hover_id, cx, cy, "Move Fleet")
        el.on_click=move_click    
        el.on_hover= purple_cursor
      end
    else
      med_icon_tooltip( hover_id, cx, cy, cant)

      if cant_report
        el.on_hover=hand_cursor
        el.on_click=report_click
        el.report=cant_report
      else

        el.on_hover= grey_cursor
      end
    end
 
    

  else

    bottom_frame.draw_frame()
  end


  do
    set_frame_color4(1,1,1,1)

    if(not title_covered()) 
      local str

      if ( fleet.empire==player) or player.known_empires[fleet.empire] or options.omniscient_developer
        str = fleet.formal_name
      else
        str = 'Unknown'  .. (fleet.civilians and " Transports" or " Fleet")
      end


      if fleet.empire.monster_faction =='space_harpies'
        str = 'Star Harpies'
      end

      if(not fleet.dest_star)
        if fleet.transport_dest
          local dest = fleet.transport_dest
          str..= " en route to ".. (revealed[dest] and dest.name or 'Unknown')
        else

          if revealed[fleet.src_star]
            str..= " at " ..fleet.src_star.name
          end

        end
      else
        local dest_name = (fleet.transport_dest or fleet.dest_star).name
        local dest_star = (fleet.transport_dest?.star or fleet.dest_star)
        str..= string.format(": %d turn%s from %s",fleet.eta, fleet.eta==1 and "" or "s", revealed[dest_star] and dest_name or 'Unknown Star' )
      end

      --bottom_bar.ensure_frame()





      if not SCOPE.no_fleet_title
        bottom_bar.title(str)
      end
    else
      --bottom_bar.ensure_frame()
    end

  end




  if options.display_fleet_packing_grid
    draw_fleet_occlusion_grid(packed,fleet)
  end




end


insert(_ENV,..draw_fleet_ships)

local function raw_check_drawer_caches()
	 checked_drawer_cache=true
  --print 'checking drawer caches'
  local ensure_ship_cached,ensure_planet_cached,ensure_sun_cached in drawers

  --wait_screen('checking drawer caches', |) function()  

  -- some of these caching functions appear to (rarely) crash on nvidia
  -- hardware.  however, because of the lazy logic, it shouldn't effect gameplay
  -- (though it is a doublesome bug)
  local ok, err = pcall | function()

    local t0=os.clock()
        
    for_all_view_fleets | function(fleet)
      for ship ; fleet.ships
        local _TRACE_INFO=fleet.empire.name
        ensure_ship_cached( ship )
      end
    end


    local explored = revealed_stars()
    --print( 'start planet checks',  timing_check())
    for _,star ; all_stars if explored[star]

      ensure_sun_cached(star)

      for _,planet in star:planet_list()
        
        ensure_planet_cached(planet)

        local empire in planet
        if empire and planet.orbitals	
          for _,orbital ; planet.orbitals?
              ensure_ship_cached( orbital)
          end
        end
      end
    end end

    -- print( 'done planet checks', timing_check())


    --print('caches checked',os.clock()-t0)

  end

  if not ok
    local log_data =  format("thread=%s, context=%s",
      inside_main_thread() and 'main' or 'wait',
      inside_main_rendering_context() and 'main' or 'bk')

    log_data..= '\n\nquiet error in fleet caching:'..tostring(err)

    log_error | log_data
    OutputDebugString | log_data

  end

  --ensure_drawer_cache("Earth")
  --ensure_drawer_cache("Jupiter")
  --ensure_drawer_cache("Ganymede")
end

function check_drawer_caches()


  if not checked_drawer_cache
		if not using_qgl(_ENV)
		  raw_check_drawer_caches()
		else
 			wait_screen(build_wait_code_v2() , raw_check_drawer_caches)  
		end
  end

end





end