local orange_text_color = color_param 'ff4f19'
local red_text_color = color_param 'ff1c1c'
local yellow_text_color = color_param 'ffef0c'
local green_text_color = color_param '00ff0c'

local pen_colors = color_table {
  orange = 'ff4f19',
  red = 'ff1c1c',
  yellow = 'ffef0c',
  green = '00ff0c',
  white = 'ffffff',
}

local function set_pen(color)
  set_frame_color4 | pen_colors[color]()
end


UI_File | function(_ENV)

function destroy_planet_click(el)
  local report in el!
  order.destroy_planet(report)
  Sounds.STELLAR_SURGE_BEAM()
  Sounds.PLANET_DESTROYED(delay=6)
end

function launch_invasion_click(el)
  local report in el!
	wait_screen(build_wait_code_v2(), order.launch_invasion, report)
  
	if report.invasion_results.is_conquest
    Sounds.PLANET_INVADE_SUCESS()
  else
    Sounds.PLANET_INVADE_FAILED()
  end
 
end

function launch_raid_click(el)
  order.launch_raid(el.report)

  if next | el.report.invasion_results.defenders_captured
    Sounds.PLANET_INVADE_SUCESS()
  else
    Sounds.PLANET_INVADE_FAILED()
  end
end

local _ENV = ensure_property_env(_ENV,'ground_combat_pane')

local damage_ani_duration=1
damage_ani_time = MarkedAlphaState (1 / damage_ani_duration)

function new_frame.ground_ani_update()
  damage_ani_time.update()
end

depends 'SummaryFrame'
depends 'PopIcons'
depends 'GroundUnits'


fade_speed=2.4

local invasion_info

local function ground_combat_noun_form(empire,race,n)
  if GroundCombatUnits[race]
    if race=='tanks'
      return n==1 and empire.tank_name:sub(1,-2) or empire.tank_name
    else 
      return n==1 and 'Raider' or 'Raiders'
    end
  end
end

add_resize_callback | function()

  local mx,my,mw,mh = center_screen_rect()
  local master_w,master_h = 950,719
  local x,y,w,h = round_all |  box_grow( .5*(master_w-mw), .5*(master_h-mh),  mx,my+14,mw,mh  )

  ground_combat_pane.dont_block_bottom_titles= (y-my) > 20

  -- cases where we need to fit the screen tightly
  if (h>mh or y<(my+4) )
    h=mh-6
    y=my+4
    --print('trouble!',..h)
  end

  local lx,ly,lw,lh = leftscreen_rect()
  local min_x=lx+lw+8

  local text_context = new_text_rendering_context { 
    name="ground_combat",
    default_bold_key='vera_bold15',
    default_font_key = 'vera15',
  }

  local lazy_word in text_context!

  -- shift over slightly if doing so will make it possible to fit
  -- the leftside pane.  (this case matters in 4x3 aspect
  -- ratios.)
  if min_x > x and ( min_x - x < 30 )
    x=min_x
  end

  allow_leftside_report = catch | min_x <= x

  right_rebuilt_flag=nil
  left_rebuilt_flag=nil


  local lazy_word_render in text_context

  local function small_word(...)
    return lazy_word_render ('vera17', shorthand_format | format(...) )
  end


  local function smaller_word(...)
    return lazy_word_render ('vera14', shorthand_format | format(...) )
  end

  local function bigger_word(...)
    return lazy_word_render ('vera24', shorthand_format | format(...) )
  end


  function on_open(report)
    --print('opening',..report)
    open_report=report!
    invade_mode=nil
    right_rebuilt_flag=nil
    left_rebuilt_flag=nil

  end

  esc_handler = function()
    --print('i got closed!')
    select_map_key(nil)
    --select_next_report_or_close()
  end


  local text_button = restore_color | function(rect_fun,text)
    local word = lazy_word_render('race_selection_title',text)
    local x,y,w,h =  SummaryFrame[rect_fun](x,y,w,h)
    small_rounded_rect(x,y,w,h)
    if FLAG.invalid_frame
      local g=.6
      set_frame_color4(.6,.6,.6,1)
    end
    word.draw_hcentered(x+w*.5,y+11) 
    return x,y,w,h    
  end

 -- local right_button = partial_application(text_button,'right_button_rect')

  insert(ground_combat_pane,..text_button,..text_context)

  local tip_element = new_tip_maker { 
    style = 'draw_center_left', 
    dy=76*.5, 
    dx=76+15}

  local left_icon = newproxy | weak_memoizeNN | function(icon) return function()
    return element | notification_icons.misc[icon].draw |  SummaryFrame.left_icon_center(x,y,w,h)
  end end

  local right_icon = newproxy | weak_memoizeNN | function(icon) return function()
    return element | notification_icons.misc[icon].draw |  SummaryFrame.right_icon_center(x,y,w,h)
  end end

  function set_desc_text()
  end

  --[[
  local right_icon =  weak_memoizeNN | function(icon,tip) 
    if tip
      tip = tip_element | tip
    end

    return hover_id_fun | function()
      return SCOPE_call( text_width=185, (tip or element), |) 
        notification_icons.misc[icon].draw |  SummaryFrame.right_icon_center(x,y,w,h)
    end 
  end
  --]]

  local left_button = newproxy | weak_memoizeNN | function(text) 
    local button = partial_application(text_button,'left_button_rect')
    return hover_id_fun | function()
      return element | button(text)
    end
  end

  local right_button = newproxy | weak_memoizeNN | function(text) 
    local button = partial_application(text_button,'right_button_rect')
    return hover_id_fun | SCOPE_fun('bk_color_key','light_bk_color',|) function()
      return element | button(text)
    end
  end

  local right_inset_button = newproxy | weak_memoizeNN | function(text) 
    local button = partial_application(text_button,'right_inset_button_rect')
    return hover_id_fun | SCOPE_fun('bk_color_key','light_bk_color',|) function()
      return element | button(text)
    end
  end

  --[[
  local right_long_button = newproxy | weak_memoizeNN | function(text) 
    local button = partial_application(text_button,'right_long_button_rect')
    return hover_id_fun | SCOPE_fun('bk_color_key','light_bk_color',|) function()
      return element | button(text)
    end
  end

  local right_upper_long_button = newproxy | weak_memoizeNN | function(text) 
    local button = partial_application(text_button,'right_upper_long_button_rect')
    return hover_id_fun | SCOPE_fun('bk_color_key','light_bk_color',|) function()
      return element | button(text)
    end
  end
  --]]



  local left_bk = catch | SummaryFrame.left_bk_rect(x,y,w,h)
  local right_bk = catch | SummaryFrame.right_bk_rect(x,y,w,h)


  local left_inset_button, left_inset_rect do
  

    local x,y,w,h=left_bk()

    left_inset_rect = function(row)
      return x+w-190,
        y+h-80-(72)*(row-1), 
        180,46
    end



    left_inset_button = uncurry | weak_memoizeNN | function(row)
      local rx,ry,rw,rh = left_inset_rect(row)

      return hover_id_fun | SCOPE_fun('bk_color_key','light_bk_color',|) restore_color| function(glyph, text)
          local word = lazy_word_render('orig32',shorthand_format | text)


          small_rounded_rect (rx,ry,rw,rh)

          SystemGlyphs[glyph].draw_at_pt(24,rx+7,ry+14)


          if FLAG.invalid_frame
            local g=.6
            set_frame_color4(.6,.6,.6,1)
          end

          word.draw_hcentered(rx+rw*.5+7,ry+11)

          return element(rx,ry,rw,rh)
      end
    end
  end


  local vbox_text_context = new_text_rendering_context { 
    name="ground_combat_vboxes",
    default_bold_key='vera_bold15',
    default_font_key = 'vera15',
  }
  local left_vboxes = framed_scrolling_vboxes {
    text_context=vbox_text_context,
    rect = left_bk,
  }

  local right_vboxes = framed_scrolling_vboxes {
    text_context=vbox_text_context,
    rect = right_bk,
  }


  local normal_draw, chibi_draw in drawers.PopIcons
  local chibi_scale = .45


  local indent0=28-5
  local number_br0=28+10
  local text_shift1 = 40
  local vspace1 = 30
  local chibi_yshift=0
  local indent1= 140

  local number_br = 145
  local unit_type_yshift = 47 
  local number_yshift=45

  local function get_invade_report()
    local invade_report = open_report
    if open_report.type=='invasion'
      invade_report=open_report.their_report
    end
    return invade_report
  end

  local function build_left_bombard(vbox_list)
    local invade_report = get_invade_report()

    local planet,empire in open_report!
    local star in planet
    local _,_,w,h=left_bk()

    invasion_info=invade_report:get_invasion_info()

    local defenders,militia,attackers,attacker_ratings in invasion_info

    local function draw_attacking_pop(y,race,n,rating)


        local _TRACE_INFO = race
        chibi_draw(race,indent1-68,y+chibi_yshift)

        local noun= ground_combat_noun_form(empire,race,n) or 'Militia'
        local pow = race:match 'raiders' and 'RAID_POW' or 'COMBAT_POW'  

        local num0 = bigger_word(tostring | n)
        num0.draw(number_br-num0.w,y+number_yshift)

        small_word(noun).draw(number_br,y+unit_type_yshift)

        local num1 = bigger_word(tostring | rating)

        bigger_word("%g<%s>",rating,pow).draw(number_br-num0.w,y+number_yshift-vspace1)
    end

    local fleet_info = weak_memoizeNN | function()
      local total_bombard = 0
      local bombs_dropped = invade_report.bombs_dropped or 0
      local military_ships = {}

      for _,fleet in pairs(empire.fleets) 
        if at_star(fleet)==star
          for ship ; fleet.ships 
            total_bombard+=ship.bombard_power
            if ship.bombard_power > 0
              military_ships[ship]=true
            end 
          end
        end
      end

      local representative_ship = get_best_key(military_ships, | ) function(a,b)
        return lexical_compare(
          a.bombard_power,b.bombard_power,
          a._ikey,b._ikey
        )
      end

      --total_bombard = ceil(total_bombard)

      return {
        ..total_bombard, ..bombs_dropped,
        bombs = total_bombard - bombs_dropped,
        --..military_ships,
        ..representative_ship,
        n_military_ships = nkeys(military_ships)
      }   
    end


    local ordered_attackers = weak_memoizeNN | function(attackers)

      local push, rval = pusher {}

      for race,p in rpairs(attackers,|) function(r0,r1)
          -- put raiders on the bottom, tanks at the top, and otherwise
          -- order by number of attackers
          return lexical_compare(
            race_to_unit_type(r0)=='raider' and 1 or 0,  race_to_unit_type(r1)=='raider' and 1 or 0, 
            race_to_unit_type(r1)=='tanks' and 1 or 0,  race_to_unit_type(r0)=='tanks' and 1 or 0, 
            attackers[r1],attackers[r0],
            r0,r1
          )
        end

        push {
          ..race,
          ..p,
        }

      end

      return rval
    end

    local draw = safe_requests | function()
      local planet_s=w*.55

      local planet_x = w -planet_s-10
      local planet_y = h - planet_s-30

      local bombs,total_bombard, bombs_dropped,n_military_ships,representative_ship in fleet_info()
      local y=h-100


      if representative_ship
--      print(..representative_ship)
        local request_ship_draws,rect_fit in representative_ship.drawer_table
        
        local box_w=100
        local box_x=20
        local box_y=y-20
        local x,y,s=rect_fit(representative_ship,box_x,box_y,box_w,box_w) 
        --glBox(x,y,s,s)
        local transform = new_request_transform( rect_fit(representative_ship,box_x,box_y,box_w,box_w) )
        
        request_ship_draws(transform,representative_ship)
        execute_draw_requests()
      end

      --print(..bombs,..total_bombard,..bombs_dropped)

      local num0 = bigger_word(tostring | n_military_ships)
      local num0x=number_br-num0.w
      num0.draw(num0x,y+number_yshift)

      --small_word("%d Ships",n_military_ships).draw(indent1,y+text_shift1)
      small_word('Ships').draw(number_br,y+unit_type_yshift)

      --lazy_word_render ('vera24', shorthand_format | format(...) )

      lazy_word_render('vera20', shorthand_format | 
        format( "%s/%d<BOMBS>", tostring | (  bombs ==0 or bombs >= 1) and (floor | bombs) or '~', 
          floor | total_bombard ) ). draw(num0x,y+number_yshift-vspace1)

--      glBox(0,0,planet_s,planet_s)

--[[

      drawers[planet.drawer].request_planet_draws(transform,planet)

      --]]


      local dy=2

      for i,info in ipairs | ordered_attackers | attackers
        local race,p in info
        local unit_type = race_to_unit_type(race)
        local attack,defense = attacker_ratings[unit_type](race)

        draw_attacking_pop(h-dy*100,race, p, attack )
        dy++
      end

      --if attackers.tanks
      --  draw_attacking_pop(h-dy*100,'tanks',attackers.tanks,open_report : invading_armor_rating())    
      --end

      --[[
      for race, p in spairs | attackers
        draw_attacking_pop(h-dy*100,race,p)
        dy++
      end
      --]]



    end
        
    vbox_list.add_vbox(draw,h)
  end




  local function build_right_bombard(vbox_list)

    local invade_report = get_invade_report()
    local planet in invade_report!
    local iconx=180

    local save_to_color = color_table {
      ground_shield_save = '45c4ee',
      tank_armor_save= 'f0ee33',
      fractional='f0ee33',
      ground_bunker_save = 'b8c7cb',
      destroyed = 'ff0000',
    }

    local draw_hit = restore_color | function(hit,alpha, x,y,w,h)
      local saved in hit
      --print(hit.saved)
      local r,g,b = save_to_color[hit.saved or 'destroyed']()
      set_frame_color4(r,g,b, .8*alpha )
      glBox(x,y,w,h)
    end

    local get_draw_damage_overlays = strong_memoizeNN | function(for_race,y)
      return function(alpha, report, hit)
        if _undefined(hit) return end
        local race in hit
        if report==open_report and for_race==race
          --draw_hit(alpha, |) centered_box(iconx,y+40, 40, 40)
          draw_hit(hit,alpha,10,y-5,150,70)
        end
      end
    end

    if invade_report==open_report
      local bombs in invade_report
      if bombs and bombs[#bombs]
        damage_ani_time.set_to_1( invade_report, bombs[#bombs] )
      end
    end

    local _,_,w,h=right_bk()

    local bombed_planet = {
      ..planet.empty_slots,
      destroyed=copy(open_report.destroyed?) 
    }
    for i, field in ipairs(generic_improvement_list)
      bombed_planet[field] = planet[field]  + ( open_report.destroyed?[field] or 0)
      bombed_planet.empty_slots-=( open_report.destroyed?[field] or 0)
      local level = planet_field_to_improvement[field]:lower() .. '_level'
      bombed_planet[level]=planet[level]
    end


    local function draw_structures(y)
      --draw_defense_icons(planet,w *.15,120,w*.8,22)
      small_word('%d Structures',planet.total_improvements).draw(indent0,y+30)
      draw_improvement_icons(bombed_planet,w*.15,y,w*.8,22)      
    end


    local function draw_defending_pop(y,race,n,rating)
--        local n = pop_round | planet.pop_and_tanks[race]

      chibi_draw(race,iconx,y+chibi_yshift)

      local noun= ground_combat_noun_form(open_report!.defending_empire,race,n) or 'Militia'
      local num0 = bigger_word(tostring | n)

      num0.draw(number_br0-num0.w,y+number_yshift)
      small_word(noun).draw(number_br0,y+unit_type_yshift)

      --small_word('%d %s',n,noun).draw(indent0,y+text_shift1)
      local num1 = bigger_word(tostring | rating)

      bigger_word("%g<COMBAT_POW>",rating).
        draw(number_br0-num0.w,y+number_yshift-vspace1)


      --local damage_alpha = damage_ani_time.alpha(open_report, race)
      damage_ani_time.call_for_keys(get_draw_damage_overlays ( race, y+chibi_yshift) )
    end

    local defenders, civilians, defender_ratings in invade_report : get_invasion_info()!
    
    
    civilians = copy | civilians

    -- merge unhappy and happy citizens for the purposes of
    -- civilian chibi display
    for race,n ; civilians
      if get_race_suffix(race)=='unhappy'
        local base = remove_race_suffix | race
        civilians[base]=n+(civilians[base] or 0)
        civilians[race]=nil
      end
    end

    local ncivilians = nkeys(civilians)
    local civilian_list = keys(civilians)
    table.sort(civilian_list,|) function(a,b)
      return lexical_compare(
        civilians[a],civilians[b],
        a,b)
    end

    local function draw_civilians(y)
--        local n = pop_round | planet.pop_and_tanks[race]
        
      if ncivilians==0
        return
      end

      local noun = 'Civilians'

      small_word(noun).draw(indent0,y+text_shift1)
      small_word(pop_string( sum(civilians))).draw(indent0,y+text_shift1-vspace1)
        
      local offsets = pop_art_chibi_offsets[nkeys(civilians)]
      if not offsets
        offsets=pop_art_chibi_offsets[#pop_art_chibi_offsets]
      end

      for i,race in ipairs | civilian_list 
        local offset = offsets[ 1 + (i-1) % #pop_art_chibi_offsets ]
        local dx,dy=unpack(offset)
        dx,dy*=1.4
        chibi_draw(race,dx+iconx,dy+y+chibi_yshift)
        i++
      end
    end

    local all_defenders = copy | defenders
    for i,bomb in ipairs | invade_report.bombs?
      if bomb.cat == 'defenders' and not all_defenders[bomb.race]
        all_defenders[bomb.race] = 0
      end
    end

    local draw = safe_requests | SCOPE_fun(..planet,|) function()
      local planet_s=w*.47

      local planet_x = w -planet_s-10
      local planet_y = h - planet_s-30

--      glBox(0,0,planet_s,planet_s)

      local transform = new_request_transform( planet_inset_transform(planet_x,planet_y,planet_s) )
      drawers[planet.drawer].request_planet_draws(transform,planet)
      execute_draw_requests()

      local dy=1

      --[[
      if (defenders.tanks or 0) > 0
        draw_defending_pop(h-dy*100,'tanks',defenders.tanks,planet.armor_rating)
        dy++     
      end
      --]]

      for race, p in spairs | all_defenders
        local unit_type = race_to_unit_type(race)
        local attack,defense = defender_ratings[unit_type](race)

        draw_defending_pop(h-dy*100,race, p, defense or attack )
        dy++
      end

      draw_civilians(h-dy*100)
      dy++

      draw_structures(100)

    end

    vbox_list.add_vbox(draw,h)

    --

  end



  local function add_forces(vbox_list, force, destroyed, icon_empire, show_captured, saved)
    local planet,invasion_results in get_invade_report()!

    local new_double_boxes = double_box_maker {
      box_h=
        invasion_results?.is_raiding_mission and 180
        or 230,
      box_xseps = {10,10,10},
      box_ysep = 10,
    }

    if invasion_results?.is_raiding_mission
       vbox_list.add_vspace(5)
    end

    local box_w,add_box,box_h in new_double_boxes(vbox_list)
    local added=0

    for unit_type, info in spairs | force
      local n,attack,defense,race in info
      if n==0 goto next end
      local destroyed = destroyed[unit_type]
      local word1 = small_word("%d Brigade%s",n, (n==1) and '' or 's')
      local word2

      if FLAG.captured_slave_case
        word1=small_word | pop_string | n
      elseif invasion_results.is_raiding_mission and not GroundCombatUnits[race]
        race=dominant_enslaveable_pop( invasion_results.pre_raid_pop, race)
      end

      --destroyed=1
--      local stats = small_word("%dATTACK", attack)

      local pen_color = 'orange'

      local saved_text
      if next | saved?[race]?
        local v=saved?[race]?
        local text,pushf in new_text_pusher()
        if v.ground_shield_save
          pushf('+%d Shields',v.ground_shield_save)
        end
        if v.tank_armor_save
          pushf('+%d Armor',v.tank_armor_save)
        end
        saved_text=text(', ')
      end

      if (destroyed or 0) > 0
        if show_captured and (destroyed < n)
          word2=small_word("%d Lost, %d Captured",destroyed,n-destroyed)
        else
          word2=small_word("%d Lost%s",destroyed,saved_text and format(' (%s)',saved_text) or '' )
        end
      else
        --print(..race,..unit_type,show_captured)
        if show_captured and not GroundCombatUnits[race]
          word2=small_word('%d Captured',n)
        else

          -- this capture case happens in slave raids, when "captures"
          -- are actually taken as slaves, rather than incorperated into
          -- your empire
          if info.captured
            word2=small_word('%d Captured',info.captured)
            --pen_color='orange'    
          else
            word2=small_word("No Losses%s",saved_text and format(' (%s)',saved_text) or '' )
            pen_color='green'   
          end

   
        end
      end

      if FLAG.captured_slave_case
        word2=nil
      end



      add_box | SCOPE_fun(..icon_empire,|) restore_color | function()

        drawers.PopIcons.normal_draw(race,0,30,box_w,box_h-20)

--        stats.draw(box_w,box_h)
        local x = word1.draw_hcentered(box_w*.5,18) 
        
        --local x,y,w,h = stats.draw(x,0)
        --set_pen 'orange' word2.draw(x+w+2,0)
        
        if word2
          set_pen(pen_color) word2.draw_hcentered(box_w*.5,0)
        end
      end

      added++

      ::next::
    end

    return added
  end

  local function collect_captured_slaves(captured_slaves)
    --for r in
    local push, captures = pusher {}
    
    for race,n ; captured_slaves

      --print('adding captured',race,n)

      push {
        ..race,
        ..n,
      }

    end
     
    --print(t2s | captured_slaves)
    return captures
  end

  local build_left_combat do

    local _,_,w,h=left_bk()
 
    build_left_combat = function (vbox_list)
      local invade_report = get_invade_report()
      local invasion_results in invade_report

      if invasion_results?.is_raiding_mission
        local th = vbox_list.add_centered_title | lazy_word_render('secret50','Captured Slaves')
        local captures = collect_captured_slaves(invasion_results.captured_slaves)

        if next | captures
          FLAG_call('captured_slave_case',
            add_forces,vbox_list,
              captures,{},
              invade_report.empire,false,{})
                  vbox_list.add_vspace(-20)
        else
           vbox_list.add_centered_word(1.2,.1,small_word '(None)')
                   vbox_list.add_vspace()
        end

      end

      local title = invade_report==open_report and 'Our Forces' or 'Enemy Forces'
      if (open_report.type == 'revolt') or (open_report.type=='revolt_failed')
        title = 'Rebel Forces'
      end

      local th = vbox_list.add_centered_title | lazy_word_render('secret50',title)
      local attackers_destroyed,attacking_force,attacker_saves in invasion_results!

      add_forces(vbox_list,attacking_force,attackers_destroyed,invade_report.empire,false,attacker_saves)
    end

  end

  local build_right_combat do
    local _,_,w,h=right_bk()

    build_right_combat = function(vbox_list)
      local invade_report = get_invade_report()

      local title = invade_report==open_report and 'Enemy Forces' or 'Our Forces'

      if (open_report.type == 'revolt') or (open_report.type=='revolt_failed')
        title = 'Loyal Forces'
      end

      vbox_list.add_centered_title | lazy_word_render('secret50',title)
      local invasion_results in invade_report!
      
      local civilian_casualties, 
        defending_force, 
        defenders_destroyed, 
        improvements_destroyed in invasion_results!

      local defenders_captured in invasion_results


      if defenders_captured
        -- remove captured slaves from the defending force
        defending_force=copy | defending_force
        for race,info ; defending_force
          local n in info
          if defenders_captured?[race]
            if defenders_captured[race]>=n
              defending_force[race]=nil
            else
              info.n-= defenders_captured[race]
              info.captured=defenders_captured[race]
            end
          end
        end
      end

      if 0 == add_forces(vbox_list,defending_force,defenders_destroyed,
              open_report.defending_empire, invasion_results.is_conquest)
      
        vbox_list.add_centered_word(1.2,.1,small_word '(None)')
      end

      if next(civilian_casualties) or next(improvements_destroyed)
        vbox_list.add_centered_title | lazy_word_render('secret50','Collateral Damage')

        if next | civilian_casualties
          vbox_list.add_text_vbox | format("%s Civilians Killed",pop_string | sum(civilian_casualties))    
        end

        if next | improvements_destroyed
          local n=sum(improvements_destroyed)
          vbox_list.add_text_vbox | format("%d Improvement%s Destroyed", n, n==1 and '' or 's')
        end

      end


    end
  end

  function right_vboxes.rebuild_if_needed()
    if right_rebuilt_flag
      return
    end
    right_rebuilt_flag=true

    right_vboxes.clear()

    if invade_mode
      build_right_combat(right_vboxes)
    else
      build_right_bombard(right_vboxes)
    end
    right_vboxes.add_vspace(h*.1)
  end

  function left_vboxes.rebuild_if_needed()
    if left_rebuilt_flag
      return
    end
    left_rebuilt_flag=true

    left_vboxes.clear()

    if invade_mode
      build_left_combat(left_vboxes)
    else
      build_left_bombard(left_vboxes)
    end

    left_vboxes.add_vspace(h*.1)

  end

  -- add simple scroll handling for the two vbox lists
  -- TODO: attack some sort of scroll-position visual to all
  -- this.
  push_zoom_handler | function(dz)
    if midscreen_pane==_ENV
      return right_vboxes.scroll_area.handle_zoom_in_rect(dz,right_bk())
    end
  end

  push_zoom_handler | function(dz)
    if midscreen_pane==_ENV
      return left_vboxes.scroll_area.handle_zoom_in_rect(dz,left_bk())
    end
  end

  order_callback 'ground combat pane' | function()
    left_rebuilt_flag=nil
    right_rebuilt_flag=nil
  end




  local raiding_option, draw_raid_odds, draw_invade_odds do
    local x,y,w,h = SummaryFrame.center_inset(x,y,w,h)
   
    local chibi_glyph_scale = 1.6

    local function format_est(est)
      local mean,var,v0,v1 in est!
      local range = 2*sqrt(var) -- look within 2 standard deviations

      local low = max(v0,round | mean-range)
      local high = min(v1,round | mean+range)

      if low==high
        return tostring(low)
      end

      return format("%d-%d",low,high)

    end

    local get_estimate_word = weak_memoizeNN | SCOPE_fun(..chibi_glyph_scale,|) function (defeated,empire)

      if not next | defeated
        return small_word 'None'
      end

      local text,pushf,lines in new_text_pusher()
      local species in empire

      if defeated.tanks
        local est = format_est | defeated.tanks
        if est~='0'
          pushf("%s%s",est,SCOPE_call(icon_empire=empire,drawers.GroundUnits.string_shorthand) )
        end
      end

      if defeated.militia
        local est = format_est | defeated.militia
        if est~='0'
          pushf("%s<CHIBI_%s>",est,species:upper())
        end
      end

      return small_word | next(lines) and text(',  ') or 'None'
    end

    raiding_option = weak_memoizeNN | function(invade_report)    
      for race ; invasion_info.attackers
        if race:match 'raiders'
          return true
        end
      end
    end



    local estx=87

    local function set_odds_pen(conquest_chance)
      if conquest_chance < 25
        set_pen 'red'
      elseif conquest_chance < 50
        set_pen 'orange'
      elseif conquest_chance < 75
        set_pen 'yellow'
      else
        set_pen 'green'
      end
    end

    do


      local get_raid_estimate_word = weak_memoizeNN | SCOPE_fun(..chibi_glyph_scale,|) function (defeated,empire,planet)

        if not next | defeated
          return smaller_word 'None'
        end

        local text,pushf,lines in new_text_pusher()
        local species in empire

        if defeated.tanks
          local est = format_est | defeated.tanks
          if est~='0'
            pushf("%s%s",est,SCOPE_call(icon_empire=empire,drawers.GroundUnits.string_shorthand) )
          end
        end

        if defeated.raider
          local est = format_est | defeated.raider
          if est~='0'
            pushf("%s<CHIBI_MARAUDER_RAIDER>",est)
          end
        end


        if defeated.militia


          local est = format_est | defeated.militia
          if est~='0'


            if planet
              local pop = planet:merged_enslavable_pop()
              local max = get_max_entry(pop)
              species = remove_race_suffix(max)..'_slave'
              print(..species)
              --local
            end

            pushf("%s<CHIBI_%s>",est,species:upper())
          end
        end

        return smaller_word | next(lines) and text(',  ') or 'None'
      end


      local dx,dy,dw,dh = left_inset_rect(5)
      dh=72
      dy-=10

      local y0=dy+dh-24
      local y1=dy+dh-24-30
      local estx=dx+118


      draw_raid_odds = restore_color | function()
        if open_report?.type~='orbit'
          return
        end


       small_rounded_rect(dx,dy,dw,dh)
        --glBox(dx,dy,dw,dh)
        set_pen 'green'
        smaller_word('Est. Captures: ').draw(dx+8,dy+dh-24)

        smaller_word('Est. Casualties: ').draw(dx+8,dy+dh-24-30)

        set_pen 'white'


        local estimate = open_report:estimate_raid_result()
   
       
        get_raid_estimate_word(estimate.defender_estimates,
          open_report.empire,open_report.planet).draw(estx,y0)

        get_raid_estimate_word(estimate.attacker_estimates,
          open_report.empire).draw(estx,y1)

        --smaller_word('Est. Casualties: ').draw(dx+8,dy+dh-24-30)
      end
    end

    draw_invade_odds = restore_color | function()
      --set_frame_color4(1,1,1,.2)

      --glBox(x,y,w,h)
      local estimate = open_report:estimate_casualties()
      local conquest_chance in estimate!

      local y1=y+h-26
      conquest_chance=floor(conquest_chance*100)

      set_odds_pen(conquest_chance)
      small_word('Chance to Conquer: %d%%',conquest_chance).draw_hcentered(x+w*.5,y1)

      local ty=y1-18

      local cx = x+w*.4
      local dx = 7
      local ty=y1-30
      small_word('Est. Casualties:').draw_br(cx-dx,ty)
      small_word('Attacker:').draw(cx+dx,ty)

      set_pen 'white' get_estimate_word(estimate.attacker_estimates,
        open_report.empire).draw(cx+dx+estx,ty)
      ty-=30
      
      set_odds_pen(conquest_chance) small_word('Defender:').draw(cx+dx,ty)

      set_pen 'white' get_estimate_word( (conquest_chance>=75) and estimate.conquered_estimates or estimate.defender_estimates,
        open_report.defending_empire or
        open_report.empire
        ).draw(cx+dx+estx,ty)
    end
  end

  local get_title_render = weak_memoizeNN | function (...)
    return lazy_word_render('secret66', format(...) )
  end

  insert(ground_combat_pane,..get_title_render,..lazy_word_render)

  draw = restore_color | SCOPE_fun(..chibi_scale,|) function()
    set_frame_color4(1,1,1,1)

   -- glBox | right_bk()

    if _undefined(open_report) or open_report?.removed or (not open_report) or (open_report?.planet?.destroyed)
      open_report=nil
      if midscreen_pane==ground_combat_pane
        close_midscreen_pane()
      end
      return
    end

    local invade_report = get_invade_report()

    invade_mode=invade_report.invasion_results and true or false

    element(x,y,w,h)

    --print(..open_report)

    local info = report_to_drawtable(open_report)

    --print(..enhanced_info,..open_report)

    if ( (not info) or (selected_map_key~=open_report) )
      and (midscreen_pane==ground_combat_pane)
      close_midscreen_pane()
    end

     SummaryFrame.draw_bk(x,y,w,h)

     left_vboxes.clipped_draw()
     right_vboxes.clipped_draw()

     local name in open_report.planet
     
     SummaryFrame.draw_frame( x, y, w, h)

     local title_render 
     
     if (open_report.type=='revolt') or (open_report.type=='revolt_failed')
       title_render = get_title_render | format("%s Revolted",name)
     elseif get_invade_report().invasion_results?.is_raiding_mission

        title_render = get_title_render | format("Raid on %s",name)
     else
       title_render = get_title_render | invade_mode and
            format("Ground Combat on %s",name) or
            format("Planetary Bombardment at %s",name)

     end

     SummaryFrame.draw_title( title_render,x,y,w,h) 

     right_icon.tank() 
     left_icon.bombard()

     do
      if invade_mode
        
        insert(left_button.CLOSE (),
          on_click=select_next_report_or_close,
          on_hover=purple_cursor,
        )

        insert(right_button.DISMISS(),
          on_click=on_dismiss_report,
          report=open_report,
          on_hover=purple_cursor,
        )


        local lower_y=SummaryFrame.center_ycenter(x,y,w,h)
        local invasion_results in invade_report

        local word

        if invasion_results.is_conquest

          if (open_report.type == 'revolt') 
            word = lazy_word_render('secret_rough76','Planet Lost') 
          elseif invade_report==open_report 
            word = lazy_word_render('secret76','Planet Conquered')
          else
            word = lazy_word_render('secret_rough76','Planet Lost')
          end         
        elseif invasion_results.is_raiding_mission

          if next(invasion_results.defenders_captured)
            word = lazy_word_render('secret76','Slaves Captured!')
          else
            word = lazy_word_render('secret_rough76','Raid Repelled')
          end

        else
         if (open_report.type=='revolt_failed')
            word = lazy_word_render('secret76','Planet Held') 
         elseif invade_report==open_report 
            word = lazy_word_render('secret_rough76','Invasion Failed')
          else
            word = lazy_word_render('secret76','Invaders Repelled')
          end   
        end

        word.draw_centered(x+w*.5,lower_y)

      else

        do
          local y=y+120

          do
            local word = lazy_word_render('vera_bold18',open_report.empire.short_name)
            local x=x+40

            pixel_hline(2,x+1,y-3,word.w-2)

            word.draw( x, y)
          end
          
          local word = lazy_word_render('vera_bold18',open_report.defending_empire.short_name)
          local x =x+w-40-word.w
          pixel_hline(2,x+1,y-3,word.w-2)
            word.draw( x, y)

        end


        draw_invade_odds()

        insert(left_button.ABORT (),
          on_click=close_midscreen_pane,
        )

        local bombs = invade_report.available_bombs

        if next(invasion_info.attackers)
          insert( right_button.INVADE(),
            on_click=launch_invasion_click,
            report=open_report,
          )
        else
          insert( invalid_frame_call (right_button.INVADE),
            on_hover=grey_cursor
          )
        end


        if bombs>=1
          insert( left_inset_button(1, 'bombs', 'BOMBARD x 1'),
            on_click=bombard_click,
            report=open_report,
          )
        elseif bombs>0

          insert(left_inset_button(1, 'bombs', 'BOMBARD '),
            on_click=bombard_click,
            report=open_report,
          ) 

        else
          insert( invalid_frame_call(left_inset_button,1, 'bombs', 'BOMBARD'),
            on_hover=grey_cursor
          )
        end

        if bombs >= 5
          insert( left_inset_button(2, 'bombs', 'BOMBARD x 5'),
            on_click=bombard_click,
            report=open_report,
            drop_n=5,
          )
        else
          insert( invalid_frame_call(left_inset_button,2, 'bombs', 'BOMBARD x 5'),
            on_hover=grey_cursor
          )
        end

        --jem.morebomb.start
        if bombs >= 10
          insert( left_inset_button(3, 'bombs', 'BOMBARD x 10'),
            on_click=bombard_click,
            report=open_report,
            drop_n=10,
          )
        else
          insert( invalid_frame_call(left_inset_button,3, 'bombs', 'BOMBARD x 10'),
            on_hover=grey_cursor
          )
        end
        --jem.morebomb.end
        if bombs>0
          insert( left_inset_button(4, 'bombs', 'BOMBARD ALL'), --jem.changed 3 to 4
            on_click=bombard_click,
            report=open_report,
            drop_all=true,
          )
        else
          insert( invalid_frame_call(left_inset_button,4, 'bombs', 'BOMBARD ALL'), --jem.changed 3 to 4
            on_hover=grey_cursor
          ) 
        end

        if raiding_option(invade_report)

          insert( left_inset_button(5, 'raid_pow', 'RAID'), --jem.changed 4 to 5
            on_click=launch_raid_click,
            report=open_report,
          )
        
          draw_raid_odds()

        end


        if (open_report.type == 'orbit') and can_destroy_planet(open_report)

          insert( left_inset_button(6.3, 'raid_pow', 'DESTROY'),
            on_click=destroy_planet_click,
            report=open_report,
          )
        

        end




      end

   end
  end

end

end

