

UI_File | function(_ENV)

create_leftside_pane 'planet_list_pane'

local pane = planet_list_pane!

local needs_rebuild=true
do

  --add_resize_callback | function() needs_rebuild=true end
  function after_weak_clears.reset_planet_list_pane()
    needs_rebuild=true
  end

  local last_empire
  function pane.rebuild_if_needed()
    needs_rebuild = needs_rebuild or  (last_empire~=gui_player())
    if needs_rebuild
      pane.rebuild_with_old_scroll()
      needs_rebuild=false
      last_empire=gui_player()
    end
  end

end

local sorting_options = {
  'Population',
  'Activity',
  'Name',
  'Labor',
  'Coin',
  'Science',
  'Food',
  'Metal',
  'Max Pop',
  'Planet Type',

   on_select = function(key)
    needs_rebuild=pane.sort_by~=key
    pane.sort_by=key
   end,

   make_word = function(text)
     return pane.text_context.lazy_word_render( 
      (text==pane.sort_by) and'vera_bold16' or 'vera14',text)
   end,

   word_offset=function(text,i)
    return (text==pane.sort_by) and -1
   end,
   sel_y_pad=3,
}

pane.sort_by = pane.sort_by or 'Population'

ListFrame.create_pane(_ENV,|) {
  name = 'planet_list_pane',
  title = 'Planetary Report',
  tiny_prefix_icon='wrench',
  tiny_prefix_dy=3,
}

planet_list_hover_text = make_hover_text_cache { 
  name = 'planet_list_hover_text',
  text_width = 350,
  bk_color = catch(0,0,0,.75),
  draw_style = 'draw_bl_bk',
  default_font_key = 'vera14',
}

local sort_funs = {
}
function sort_funs.Population (p1,p2)
  return lexical_compare(
    p2.outpost and 0 or total_pop(p2), p1.outpost and 0 or total_pop(p1),
    p1.name , p2.name,
    p1._ikey, p2._ikey)
 end

sort_funs["Max Pop"] = function (p1,p2)
  return lexical_compare(
    p2.outpost and 0 or calc_pop_slots(p2),p1.outpost and 0 or calc_pop_slots(p1),
    p2.outpost and 0 or total_pop(p2), p1.outpost and 0 or total_pop(p1),
    p1.name , p2.name,
    p1._ikey, p2._ikey)
 end
--jem.sort.planet.type
local size_to_s = {
	Tiny = 14,
	Small = 18,
	Medium=22,
	Large=24,
	Huge=28,
}
sort_funs["Planet Type"] = function (p1,p2)
  return lexical_compare(
    p2.type, p1.type,
    size_to_s![p2.size], size_to_s![p1.size],
    p2.outpost and 0 or total_pop(p2), p1.outpost and 0 or total_pop(p1),
    p1.name , p2.name,
    p1._ikey, p2._ikey)
 end

function sort_funs.Name (p1,p2)
  return lexical_compare(
    p1.name , p2.name,
    p2.outpost and 0 or total_pop(p2), p1.outpost and 0 or total_pop(p1),
    p1._ikey, p2._ikey)
 end

 function sort_funs.Labor (p1,p2)
  return lexical_compare(
    p2.raw_hammer_production, p1.raw_hammer_production,
    p1.name , p2.name,
    p2.outpost and 0 or total_pop(p2), p1.outpost and 0 or total_pop(p1),
    p1._ikey, p2._ikey)
 end

function sort_funs.Science (p1,p2)
  return lexical_compare(
      p2.beaker_production,p1.beaker_production,
      p1.name , p2.name,
      p2.outpost and 0 or total_pop(p2), p1.outpost and 0 or total_pop(p1),
      p1._ikey, p2._ikey) 
    
end

function sort_funs.Coin (p1,p2)
  return lexical_compare(
      p2.net_coin_production, p1.net_coin_production,
      p1.name , p2.name,
      p2.outpost and 0 or total_pop(p2), p1.outpost and 0 or total_pop(p1),
      p1._ikey, p2._ikey) 
    
end

function sort_funs.Metal (p1,p2)
  return lexical_compare(
      p2.net_metal, p1.net_metal,
      p1.name , p2.name,
      p2.outpost and 0 or total_pop(p2), p1.outpost and 0 or total_pop(p1),
      p1._ikey, p2._ikey) 
    
end

function sort_funs.Food (p1,p2)
  return lexical_compare(
      p2.net_food_production, p1.net_food_production,
      p1.name , p2.name,
      p2.outpost and 0 or total_pop(p2), p1.outpost and 0 or total_pop(p1),
      p1._ikey, p2._ikey) 
    
end


do

  local project_order = {
    'refitting',
    'building_design',
    'building_ground_unit', --jem
    'building_improvement',
    'building_colony',
    'building_planet',
    'terraforming',

    --'activity',  
  }

  local inv=invert_table | project_order

  local raw_project_rank = function(planet)
    local project_type = planet: project_info()
    if not inv[project_type]
      return
    end

    return #project_order+1 - 
      (inv[project_type] or (#project_order+1) )
  end

  local function project_rank(planet)
    if planet.no_colony
      return -100
    end

    local project_type = planet: project_info()
    if not project_type
     return 200
    end

    local raw = raw_project_rank(planet)
    if not raw
      return 0
    end

    return (planet.player_set_production and 100 or 0) + raw
  end

  local activity_to_sort_key = {
    Trade = 'net_coin_production',
    Research = 'beaker_production',
    Mining = 'net_metal',
    ['City Planning']='summed_pop',
    ['Synthesize Food'] = 'net_food_production',
  }

  local function eq_project_rank(planet)
    local project_type,project_info = planet: project_info()
    if not project_type
      return planet.raw_hammer_production
    end

    if planet.producing_hammers
      return planet.hammers
    end

    if project_type=='activity'
      local key = activity_to_sort_key[project_info]
      return planet[key] or 0
    end

    return 0
  end

--jem.buycost
  local function project_buycost(planet)
    local project_type,project_info = planet: project_info()
    if not project_type
      return 0
    end
    
    local hammer_cost = planet.current_production_cost
    if not hammer_cost
      return 0
    end

    local remaining = hammer_cost-planet.hammers-planet.overflow_hammers
    remaining=max(0,remaining)
    return remaining * planet.empire.coins_to_labor_rate
  end

  local function project_buyturn(planet)
    if planet:turns_to_complete() ~= 1
      return 0
    end
  end
--jem.buycost.end

--jem.projectgroup
  local function project_group(planet)
    return string.sub(planet.project_name,1,10)
  end
--jem.projectgroup.end

  function sort_funs.Activity (p1,p2)
    return lexical_compare(
      project_rank(p2),project_rank(p1),
      p2.project_type, p1.project_type, --jem
      project_group(p1), project_group(p2),
      project_buyturn(p1),project_buyturn(p2), --jem
      project_buycost(p1),project_buycost(p2), --jem
      p1:turns_to_complete(),p2:turns_to_complete(), --jem
      p1.name , p2.name,
      p2.outpost and 0 or total_pop(p2), p1.outpost and 0 or total_pop(p1),
      p1._ikey, p2._ikey)
  end
end



get_planet_list = weak_memoizeNN | function (empire,sort_by)
  
  local push, list = pusher {}
  for p ; empire.planets
    push | p
  end

  --print(v2s | empire.planets)





  local sort_by in pane!
  table.sort(list, sort_funs![sort_by]) 



  return list
end

function select_next_planet(el)
  local planet in el!
  local list = get_planet_list ( planet.empire, pane.sort_by)


  for i=1,#list
    if planet==list[i]


      for k=0,#list
        local j= ( ( i+k) % #list ) + 1
        el.planet=list[j]
        if el.planet.productive
          break
        end
      end

      Sounds.WIDGET_INCREMENT_UP()
      center_at_star(el.planet.star)
      select_map_key(el.planet.star)
      return planet_info_click(el)
    end
  end
end

function select_prev_planet(el)
  local planet in el!
  local list = get_planet_list ( planet.empire, pane.sort_by)
  for i=1,#list
    if planet==list[i]

      for k=1,#list
        local j= i-k


        if j<1
          j= #list + j
        end

        el.planet=list[j]
        if el.planet.productive
          break
        end
      end

      Sounds.WIDGET_INCREMENT_DOWN()
      center_at_star(el.planet.star)
      select_map_key(el.planet.star)
      return planet_info_click(el)
    end
  end
end


add_resize_callback | function  ()

  local tooltip_el = new_tip_maker {
    style = 'draw_center_left', 
    no_desc_text=true,
    dx=52,
    dy=16,

   -- dx=10,
   -- dy=0,
  }
  local tp_element,tooltip_id = tooltip_el('Sort By:')

  local text_bk_color=drawers.frame_colors.dark_dropdown_color

  --local text_bk_color=catch(0,0,0,1)

  sorting_options.on_choosing_draw = function()
     tooltip_activations.mark(tooltip_id)
  end

  function pane.draw_overlay_content(x,y,w,h)
      vertical_arrows.draw_small_down( x+w-67, y+h-21 )

    local el = SCOPE_call(..text_bk_color, tp_element,x+w-100,y+h-34,46+20,38)
    local dropdown_id = sorting_options



    local rx,ry,rw,rh = x+w-85,y+h-34,153,38

    dropdown_menu.register_menu(dropdown_id, sorting_options,
      rx-20,ry,rw,rh)

    el.dropdown_id=dropdown_id
    el.on_click = dropdown_menu.on_open_dropdown
    --glBox()
  end

end

function planet_list_pane.rebuild_vbox_list(vbox_list)



  local plus_form in _ENV

  local w in vbox_list
  local item_h=52
  local test_bk = restore_color | function ()
    set_frame_color4(1,1,1,.3)
    --glBox(0,0,w,item_h)
  end

  --print 'rebuilding list!'

  vbox_list.clear() 

  local lazy_word,lazy_word_hcentered,lazy_word_render in vbox_list!.text_rendering_context

  vbox_list.add_vspace(8)

  local x1=60
  local y1=item_h-20

  local size_to_s = {
		Tiny=.5,
    Small=.65,
    Medium=.75,
    Large=.88,
    Huge=1,
  }

  local make_number = strong_memoize_1_to_1 | function (i)
    
    local pt,font in fonts.orig21

    pt=max(14,round | pt*current_gl_scale())

--      print(..pt)

    local text,info = outlined_text {
      center = catch(1,1,1,1),
      outline1 = catch(0,0,0,.3),
      outline_width1=4,
      ..font,
      ..pt,
      text = tostring(i),
    }


    return info
  end

  local function planet_icon_request(planet,x0,y0)
    local s=size_to_s[planet.size]
    local max_s=28
    local x,y,w,h = box_grow( s*max_s,s*max_s, 30,30, 0,0 )

    if not options.hide_planet_icons
      --print(..y0,..y)
      request_planet(planet,x0+x,y0+y,w)

--       execute_draw_requests()
    end   
  end

  local planet_draw = safe_requests | function(planet)
     
    local s=size_to_s[planet.size]
    local max_s=28
    local x,y,w,h = box_grow( s*max_s,s*max_s, 30,30, 0,0 )


    local word = lazy_word_render(
      'orig21',
      planet.name,
      x1,y1
    )
    word.draw(x1,y1)

    if planet.productive

      local orender = make_number(display_pop(planet) )

      local image in orender
      local iw,ih = image:dims()

      local x,y = screen_position(30,y1-2)

      x,y-=iw*.5,ih*.5

    	call_with_identity_matrix(image.draw, image, round(x), round(y))

    end

    return word
  end


  local oranges = color_table {
    progress = 'f8971d',
    next_progress = '8f5909',
    empty = '28231c',
  }

  local reds = color_table {
    progress = 'd43323',
    next_progress = '741c10',
    empty = '1f1c17',
  }

  local blues = color_table {
    progress = '0360cb',
    next_progress = '094390',
    empty = '01254f',
  }

  local greens = color_table {
    progress = 'hsv: 117;83;70',
    next_progress = 'hsv: 117;90;50',
    empty = 'hsv: 117;20;14',
  }


  local trade_yellow = color_param 'c8cd00'
  local foodgoo_color = color_param 'hsva: 100; 70; 100; 100'
  local mining_color = color_param 'hsva: 18; 100; 80; 100'


  local progress_frame = restore_color | function(colors,progress,next_progress)
    local x,y,w,h=x1,18,w-x1-15,10
      
    next_progress=min(next_progress,1)

    local w0=w*progress
    local w1=(next_progress-progress)*w
    local w2=w-w1-w0


    if w0>0
      set_frame_color4(colors.progress())
      glBox | round_all(x,y,w0,h)
      x+=w0
    end

    if w1>0
      set_frame_color4(colors.next_progress())
      glBox | round_all(x,y,w1,h)
      x+=w1
    end

    if w2>0 
      set_frame_color4(colors.empty())
      glBox | round_all(x,y,w2,h)
    end

    --print(..progress,..next_progress)
  end

  local draw_production = restore_color | function (planet)

    if planet.productive
      local ptext = planet.active_project_name

      if not planet.player_set_production
        ptext..='*'
      end

      lazy_word_hcentered(
        'vera12',
        ptext,
        .5*(w-x1)+x1,4
      )

      do

        local progress, next_progress = planet:production_progress()

        if progress
          progress=min(progress,1)
        end

        local x,y,w,h=x1,18,w-x1-15,10
      
        --print(planet.name,..planet.activity)

           
        if planet.activity=='Stockpile'
          set_frame_color4(oranges.next_progress())
          glBox(x,y,w,h)
        elseif planet.activity=='Trade'
          set_frame_color4(trade_yellow())
          glBox(x,y,w,h)   
        elseif   planet.activity=='Mining'
          set_frame_color4(mining_color())
          glBox(x,y,w,h)   
        elseif planet.activity=='Synthesize Food'
          set_frame_color4(foodgoo_color())
          glBox(x,y,w,h)   
        elseif planet.activity=='Research' and progress
          progress_frame(blues,progress,next_progress)
        elseif planet.activity=='City Planning' and progress
          progress_frame(greens,progress,next_progress)
        elseif planet.activity=='Manufacture Population' and progress
          progress_frame(greens,progress,next_progress)
        elseif not (progress and next_progress)
          set_frame_color4(.6,.6,.6,1)
--            glBox(x,y,w,h)
          pixel_border(1,x,y,w,h)
        else
          local show_metal_warning in planet
          progress_frame(show_metal_warning and reds or oranges,progress,next_progress)
        end


        do -- showing all 5 numbers gets pretty spread-sheaty

          local lx = w+x1

          local spacing=4
          local plus_form = tostring

          local word = lazy_word_render('list_colored_number',plus_form | planet.net_food_production )
          set_frame_color4(0,.9,0,1)
          word.draw_br(lx,y+h+4)
          lx-=word.w+spacing


          set_frame_color4(0,1,1,1) 
          local word = lazy_word_render('list_colored_number',plus_form | planet.beaker_production )
          word.draw_br(lx,y+h+4)
          lx-=word.w+spacing    
 

          local word = lazy_word_render('list_colored_number',plus_form | planet.net_coin_production )
          set_frame_color4(1,1,0,1)
          word.draw_br(lx,y+h+4)
          lx-=word.w+spacing
          
          set_frame_color4(1,.8,0,1)
          local word = lazy_word_render('list_colored_number',plus_form | planet.raw_hammer_production  )
          word.draw_br(lx,y+h+4)
          lx-=word.w+spacing

          if options.show_metal_in_planet_list
            set_frame_color4(1,.3,0,1)
            local word = lazy_word_render('list_colored_number',plus_form | planet.net_metal  )
            word.draw_br(lx,y+h+4)
          end

        end

      end


    else

      lazy_word(
        'vera12',
        '(Outpost)',
        x1,16
      )
        
    end

  end

  local function list_planet_click(el)
    select_map_key(el.planet.star)
    planet_click(el)
    center_at_star(el.planet.star)
  end

  local function list_transport_click(el)
    center_at_star(el.planet.star)
    on_planet_transport_button(el)

  end

  for i,planet in ipairs | get_planet_list ( gui_player(), pane.sort_by)

    --
    -- i'm definitely getting a bit overly creative with shared
    -- upvalues here.
    --
    -- the use of "title" and "hover_id" both take advantage of the
    -- assumption that each draw cycle, 'draw' will be called
    -- before 'draw_hover'.  while it works fine, and is actually
    -- fairly safe -- it's also potentially pretty damn confusion.
    -- um, sorry about that ;-P
    --
    local transport_tooltip_id = {}
    local draw 
    local title_word
    local hover_id


    local draw_hover_buttons = SCOPE_fun(icon_radius_hint=12,|) function (hover_id,title_word)
       set_frame_color4(1,1,1,1)

       local x = x1+title_word.w+5
--       glBox(x,y1-2,18,18)
      -- ResourceGlyph.transport.draw_glyph(18,x,y1-2)


      local cx,cy = x+12,y1+8

      local text,can_transport = PlanetTransport.get_planet_tooltip(planet)

      local icon = can_transport and 'spaceship_colored' or 'spaceship_grey'

      local el = element | notification_icons.misc[icon].draw(cx,cy)
      el.hover_id = draw
      el.tooltip_id = transport_tooltip_id
      el.on_click = can_transport and list_transport_click
      el.planet=planet

    end

    local draw_button_tooltips = SCOPE_fun(text_bk_color=catch(0,0,0,.7),|) function (hover_id,title_word)
      set_frame_color4(1,1,1,1)
       local x = x1+title_word.w+5
      local cx,cy = x+12,y1+8
      local tooltip_id = transport_tooltip_id
      local text,can_transport = PlanetTransport.get_planet_tooltip(planet)
      draw_tooltip_cl(tooltip_id,cx+20,cy,text)

    end


    local function button_draw_fun(fun)
       if not planet_selection_mode

        local hover_id = draw
        local hover_alpha = hover_alpha_states[hover_id] or 0
        if hover_alpha > 0
          call_in_color_scope(SCOPE_frame_activation, hover_alpha, fun,hover_id,title_word)
        end
      end     
    end


    draw = restore_color | function()

      hover_id = draw
      
      if (midscreen_pane == embark_pane) or (planet_selection_mode==PlanetTransport)
        hover_id= 'TT' .. planet._recorded_name
      end

      insert(element(0,0,w,item_h),
        on_hover = hand_cursor,
        on_click = list_planet_click,
        transport_key=planet,
        ..planet,
        map_key=planet.star,
        ..hover_id
      )
      --test_bk()

      title_word = planet_draw(planet)
      draw_production(planet)


      button_draw_fun(draw_hover_buttons)

      if options.display_ai_debug_info
        local info = gui_player().ai_state?.planet_notes?[planet]
        if info
          local text = pane.text_context.lazy_rendered_text('vera10',info,200 )

          set_frame_color4(0,0,0,.5)
          local x,y=35,20
          text.draw_bl_bk(x,y)
          set_frame_color4(1,1,1,1)
          text.draw_bl(x,y)
          --error(info)
        end
      end
    end




    hover_id_to_speed[draw]=4
    
    --SCOPE_call('list_icon', true
    
    local _,vbox = vbox_list.add_vbox (draw,item_h)

    function vbox.draw_hover()

      button_draw_fun(draw_button_tooltips)

      
      if hover_alpha_states[hover_id] and is_transport_hover_id(hover_id )

        local hover_text = get_transport_hover_text(planet)
        set_frame_color4(1,1,1,1)
        if hover_text

          local cx,cy=x,y
          planet_list_hover_text.hover_text(hover_id ,40,9,
            hover_text
          --  'hi\nlets add\nmultiple lines'
          )

        end
      end
  
    end

    --[[
    vbox.draw=restore_color | function(x,y)
      print('draw',x,y)
      glColor4d(0,1,0,.6)
      glBox(0,0,100,5)
    end

    vbox.draw_hover=restore_color | function(x,y)
      print('draw_hover',x,y)
      glColor4d(0,0,1,.6)
      glBox(0,0,100,5)
    end
    --]]

    function vbox.request_bk_content(x,y) 
      planet_icon_request(planet,x,y)
    end


    vbox_list.add_vspace(4)

  end

end


end

if lua_load_complete and false



  options.hide_planet_icons=false

end