local resource_desc_text = {
	debris_ring = 'Debris Ring',
	opil='Opil Crystals',
	artifacts = 'Ancient Artifacts',
	gems= 'Rare Gems',
	rare_earths='Precious Metals',
	super_grain='Supergrain',
}

local resource_hover_text = {

  debris_ring = function(planet) 

    if true --not planet.inhabited
      return shorthand_format 'Debris Ring'
    end
  end,


  opil = function(planet) 

    if true --not planet.inhabited
      return shorthand_format 'Opil Crystals'
    end

    return shorthand_format | format('Opil: +%d<COIN>, +%d<INFLUENCE>',
      planet.opil_coin_bonus,planet.opil_influence_bonus) 
  end,

  super_grain = function(planet) 
    if true --not planet.inhabited
      return shorthand_format 'Supergrain'
    end

    return shorthand_format | format('Super Grain: +%d<FOOD>',
      planet.super_grain_food_bonus) 
  end,

  gems =function(planet) 
    if true --not planet.inhabited
      return shorthand_format 'Rare Gems'
    end

    return shorthand_format | format('Gems: +%d<COIN>',planet.gem_coin_bonus),
  end,

  artifacts = function(planet) 
    if true -- not planet.inhabited
      return shorthand_format 'Artifacts'
    end
    --JWF Note: these are confusing, as they show 0 at the start of the game and don't make clear that it's a per-building bonus. It looks like it's broken even though it's not.
    return shorthand_format | format('Artifacts: +%d <BEAKER>',planet.artifact_bonus) 
  end,

  rare_earths =function(planet) 
    if true --not planet.inhabited
      return shorthand_format 'Precious Metals'
    end

    return shorthand_format | format('Precious Metals: +%d<COIN>',planet.rare_earth_coin_bonus),
  end,

}

local get_planet_left_hovers = weak_memoize_1_to_1 | function(planet)
  local push, rval = pusher {}

  for i, resource in ipairs | planet.special_resources?
    
    push {
      icon = notification_icons.orbital_stuff[resource],
      hover_text = resource_hover_text[resource]?(planet) or capitalize | resource,
			desc_text = resource_desc_text[resource],
    }

  end

  if planet.minerals == 'Rich'
    push {
      icon = 'gem',
      hover_text = 'Mineral Rich',
			desc_text = 'Mineral Rich',
    }
  elseif planet.minerals == 'Poor'
    push {
      icon = 'shovel',
      hover_text = 'Mineral Poor',
			desc_text = 'Mineral Poor',
    }    
  end

  if planet.empire==gui_player()
    local revolt = planet:chance_to_revolt()
    if revolt > 0
      push {
        icon = 'war_sword',
        hover_text = format('%d%% Chance to Revolt!',round | 100*revolt),
      }
    end
  end

  return rval
end

UI_File | function(_ENV)

local icon_radius_hint = 18
local bottom_bar in _ENV

local overlays = myskin.PlanetOverlays(_ENV)
planet_overlays = overlays


-- {
--	Barren = "Ganymede",
-- Barren = "Mars",
-- Barren = "Hive",
--Barren = "Mercury",
--	Barren = "Hive",
--	Garden = "Earth",
--	["Gas Giant"] = "Jupiter",
--}


planet_hover_text = make_hover_text_cache { 
  name = 'planet_hover_text',
  text_width = 300,
  bk_color = catch(0,0,0,.75),
  draw_style = 'draw_center',
  default_font_key = 'vera16',
}


local function prefix_overlay_id(p,planet)
  return p..planet._recorded_name
end

function is_transport_hover_id(hover_id)
  return (type(hover_id)=='string') and hover_id:sub(1,1)=='T'
end

function row_planet_hover_id(planet)
  if (midscreen_pane == embark_pane) or (planet_selection_mode==PlanetTransport) or (planet_selection_mode==AlienPopDelivery)
    return prefix_overlay_id('T',planet)
  end
  return planet    
end

local function list_planet_overlay_id(planet)

  if (midscreen_pane == embark_pane) or (planet_selection_mode==PlanetTransport)
    return prefix_overlay_id('R',planet)
  end
  return prefix_overlay_id('S',planet)    

end

local draw_mimage, mimage_dims,set_frame_color4 in _ENV

local bar_hw


local title_fade

local draw_orbitals

local orbital_offsets do
  local yp=.05
  local dy=.3

  orbital_offsets = {
    {.8,2*dy+3*yp},
    {.8,dy+2*yp},	
    {.8,yp},

    {1.1,2*dy+3*yp},
    {1.1,dy+2*yp},	
    {1.1,yp},

    {1.4,2*dy+3*yp},
    {1.4,dy+2*yp},	
    {1.4,yp},

  }


  local function orbital_view(el)
    open_midscreen_pane(ship_status_pane,el.ship)
  end

  function draw_orbitals(planet,x,y,w,s)
    local orbitals in planet

    if(not orbitals) return end

    local _TRACE_INFO = planet.name


    for i,v in planet:orbital_list()

      local _TRACE_INFO = recorded_name(v)

      local xp,yp = table.unpack(orbital_offsets![i])

      --sprint(w)

      local w_mult = v.s>8 and .3 or .28

      --w_mult=.4

      local rect_r=1

      if v.is_rect
        rect_r=(v.s/v.h)
        w_mult*= (v.s/v.h)
      end

      request_recorded_ship_draw(v,x+xp*w,y+yp*s,s*w_mult,0)



      --glBox(x+xp*w,y+yp*s,s*w_mult,s*w_mult/rect_r)
    end

    execute_draw_requests()
  

    for i,v in planet:orbital_list()
    
      local xp,yp = table.unpack(orbital_offsets![i])

      --sprint(w)

      local w_mult = v.s>8 and .3 or .28

      --w_mult=.4

      local rect_r=1

      if v.is_rect
        rect_r=(v.s/v.h)
        w_mult*= (v.s/v.h)
      end

      if v.inactive
        SCOPE_call(
          icon_radius_hint= (s / rect_r)*w_mult*.24,
          notification_icons.misc.sleep.simple_draw,x+xp*w+s*w_mult*.9,y+yp*s+ (s / rect_r) *w_mult*.9)
      end

      local rx,ry,rw,rh = x+xp*w,y+yp*s,s*w_mult,s*w_mult/rect_r

      local el =element(rx,ry,rw,rh)

      --el.on_hover=grey_cursor
      el.on_click = orbital_view
      el.ship=v

      local ship = v
      local hover_id = ship
      el.hover_id = ship
      el.tooltip_id = ship
      draw_tooltip_cl(hover_id,rx+rw*.5,ry+rh*.1,ship.name)
      el.on_hover=help_cursor




    end


  end




end



  --request_orbitals=nil




local function contact_faction(el)

  local empire in el.planet!
  insert(el,..empire)
  select_map_key(empire)
  diplo_pane.on_contact | el

end

local function hover_button(planet, cx,cy, icon, text,on_click)
  local hover_id = planet

  -- never show on-hover buttons when we're selecting a transport
  -- destination
  if planet_selection_mode
    return
  end

  local alpha=hover_alpha_states.state[hover_id] or 0

  if type(icon)=='string'
    icon = notification_icons.misc![icon]
  end

  local el = element | SCOPE_frame_activation(alpha*frame_activation(),
    SCOPE_call,..icon_radius_hint,
      icon.draw, cx,cy)  

  local sub_tooltip_id = recorded_name(planet) .. ' - ' .. text

  draw_tooltip_cr(sub_tooltip_id,cx-icon_radius_hint-8,cy,text)

  el.on_click = on_click
  el.hover_id = hover_id
  el.tooltip_id = sub_tooltip_id
  el.planet=planet

  return sub_tooltip_id

end


function dont_transport_colonists(el)
end

local planet_font = "SecOrigCondBB"

function planets_display_width(planets)
  return nkeys(planets) * (overlays.planet_spacing + overlays.planet_w)
end

local large_dummy_planet = { size= 'Large'}

local function planet_special_refresh(el)
  local r = order.refresh_planet_scan(gui_player(), el.planet,el.special)
  check_order_callbacks()
  check_map_elements()
  el.report=r

  report_activate_and_click (el)
end 

draw_fading_planets  = safe_requests | function  (star,...)


  local no_orbitals,notitle,planets,force_x,ignore_orbits,no_planet_action,no_bar_bk,no_pop_bar in ...

  --local size_to_yshift, size_to_width in overlays

  --title_fade=fade

  local _,_,bw,bh=bottom_bar.rect()
  local cy= .5*bh



  planets = planets or star.planets


  local spacing,planet_w = overlays.planet_spacing,overlays.planet_w
  local total_w,x0

  if star

    --local planets_w=(planet_w+spacing)*4
    -- with this logic, we basically just shove the star to the far
    -- left, and all the planets to the far right.  in tex terms:
    -- {star}\hfil{planets}
    -- it's not ideal.  but, the logic we'd actually want is a bit
    -- tricky to debug and define.
    --
    -- basically, each different star drawer probably has a minimum
    -- distance from the first orbit that we'd really like to see
    -- enforced.  *but* getting that much horizontal space is a
    -- lower priority than making sure all the planets are visible.
    -- in a narrow layout -- drawing the star under the planets is 
    -- *bad*, but having some planets that you can't interact with 
    -- at all is *worse*. so, in the case that star-min-dist is
    -- large enough that satisfying that condition is impossible,
    -- the behavior that we want is probably this one -- but, in
    -- the case where min-dist can be satisfied, something like
    -- expanding in a way that keeps the planets more or less
    -- centered could be better:
    --  {star}\starspace\hfil{planets}\hfil
    -- in any case though -- that's the kind of tweaking that only
    -- matters for very wide or very narrow layouts, in 4x3 or
    -- 16x9, it's not too bad as is.
    total_w= (planet_w+spacing)*4+spacing*.5
    x0=bw-total_w



    --print(x0)
    local min_x0=450

    if x0>min_x0
      x0=min_x0
    end

    drawers.request_sun(star.type, drawers.get_sun_dims(star.type,planet_w,spacing,x0,bh) )
  else
    total_w= (planet_w+spacing)*4
  end

  

  if ignore_orbits
    total_w=(planet_w+spacing)*nkeys(planets)
  end


  if not force_x
    x0= x0 or (.5*bw- .5*total_w)
  end

  local y = cy


  local planet_row_scale in bottom_bar
  local planet_scale = planet_row_scale*260/300

  for i,planet in spairs(planets)
    
    local x= force_x or ( x0 + (planet_w+spacing)*(planet.orbit-.5) )

    if ignore_orbits
      x=( x0 + (planet_w+spacing)*(i-.5) )
    end

    --local w = size_to_width[planet.size]

    local w=planet:row_width(planet_scale)
    
    --print(w,planet.drawer)

    request_planet(planet,x-.5*w,y-.5*w+planet:row_yshift(planet_scale),w)
  
  end

  set_frame_color4(1,1,1,1)

  if not no_bar_bk
    bottom_frame.draw_bk()
  end

  clipped_box(0,0,bw,bh,execute_draw_requests)
--  execute_draw_requests()


  if not no_bar_bk
    bottom_frame.draw_frame()
  end

  x,y= x0,cy


  local action_tab,production_tab in overlays

  for i,planet in spairs(planets)

    --request_planet(type2drawer[v.type],x,y,w)

    local warning_font = "fonts/bitstream/verabd.TTF"

    set_frame_color4(1,1,1,1)

    --x+=.5*(planet_w+spacing)

    x= force_x or ( x0 + (planet_w+spacing)*(planet.orbit-.5) )

    if ignore_orbits
      x=( x0 + (planet_w+spacing)*(i-.5) )
    end

    --console(..x0)

    -- draw the pop bar and link to @PlanetInfo
    local el 
    
    -- don't create any clickable elements unless the bar is all
    -- the way faded in.

     el=element(x-.5*(spacing+planet_w),0,planet_w+spacing,bottom_bar.height)

--		 set_frame_color4(1,1,1,.6)
--		 glBox(x-.5*(spacing+planet_w),0,planet_w+spacing,bottom_bar.height )
--		 set_frame_color4(1,1,1,1)

  --console(t2s | planet)
    local hover_id = (not planet_selection_mode) and planet

    do

        local v=planet
        
        --local s=size_to_width.Large

        local x= force_x or ( x0 + (planet_w+spacing)*(v.orbit-.5) )
    
        if ignore_orbits
          x=( x0 + (planet_w+spacing)*(i-.5) )
        end

        

        local w = Planet.row_width(planet,planet_scale)
        local ys = Planet.row_yshift(large_dummy_planet,planet_scale)
        local s=Planet.row_width(large_dummy_planet,planet_scale)


        local ox, oy=x-.5*w,y-.5*s+ys
        if not no_orbitals
          draw_orbitals(v,ox,oy,w,s)
        end

        local w_mult = s>8 and .3 or .28
        local empire = gui_player()
        local offset = nkeys(planet.orbitals?)+1
        ox,oy+=w_mult*s*.5
        if not planet.empire
          for i,special in ipairs | planet.specials?
            local encounter = special[1]
            if (encounter~='removed') and ( special.scanned?[empire] )
              --print'do the thing!'

              local xp,yp = table.unpack(orbital_offsets![offset]) 
              local cx,cy=ox+w*xp,oy+s*yp
              local el = element | SCOPE_call(..icon_radius_hint,
               notification_icons.misc.exclamation.draw,cx,cy)

              offset++
              insert(el,..hover_id)
              local sub_tooltip_id = recorded_name(planet) .. ' - special_' .. tostring(i)
              local text = EncounterCallbacks[special[1]]?.on_hover_desc?(planet,special) or special[1]
              draw_tooltip_cr(sub_tooltip_id,cx-icon_radius_hint-8,cy,text)

              el.planet= planet
              el.special = special
              el.on_click = planet_special_refresh 
              el.tooltip_id = sub_tooltip_id
              
            end
          end
        end

        for i,info in ipairs | get_planet_left_hovers(planet)
          local xp,yp = table.unpack(orbital_offsets![1]) 
          local cx,cy=x-planet_w*.28,oy+s*yp-(i-1)*2*icon_radius_hint
          local sub_tooltip_id = info
          local hover_text,icon in info
          local icon = icon.draw and icon or notification_icons.misc[icon]


          --local alpha=hover_alpha_states.state[hover_id] or 0
          SCOPE_call(..icon_radius_hint,
              icon.draw_shadowed_icon, cx,cy)  



          --hover_button(planet,cx,cy,icon,hover_text)
          local el = element | centered_box(cx,cy,2*icon_radius_hint,2*icon_radius_hint)
          local tooltip_id=info
					el.desc_text in info
          el.tooltip_id = tooltip_id
					el.open_planet = planet
          
          SCOPE_call(text_width=150,
            draw_tooltip_cr,tooltip_id,cx-icon_radius_hint-4,cy,hover_text)

        end

    end



    do


      el.transport_key=planet
      el.on_click=planet_dev_click
      el.planet=planet
      el.hover_id = row_planet_hover_id(planet)

      local color = "0e335cff"
      local slots = calc_pop_slots(planet,gui_player())
      local str = tostring(slots)

--      print(el.hover_id[1])

      if hover_alpha_states [el.hover_id] and is_transport_hover_id(el.hover_id )

        local hover_text = get_transport_hover_text(planet)

        if hover_text

          local cx,cy=x,y
          planet_hover_text.hover_text(el.hover_id ,cx,cy+12,
            hover_text
          --  'hi\nlets add\nmultiple lines'
          )

        end
  
      end


      -- TODO: retire this
      if  top_transport_key==planet 
        planet_selection_mode?.on_planet_hover?(planet,el,x,overlays)
      end

			local summed_hab_pop,outpost,inhabited,productive in planet

      if productive
        local slots = calc_pop_slots(planet,planet.empire)
        color = COLORS[planet.empire.color].pop_bar or "FF00FF"
        str=string.format("%d/%d",display_pop(planet),slots)
				if summed_hab_pop > 0
					str=format("%d/%d + %d",cap_floor(planet.summed_pop),slots, summed_hab_pop)
				end
      end



      if (planet.type~='Gas Giant' and planet.type~='WarpNode') and not no_pop_bar

        if planet.inhabited
          overlays.pop_bar(x,str,color,1,planet:growth_progress() )
        elseif productive
          overlays.pop_bar(x,str,color,1, .25, .25)
				else

					--local r = 
					
          local can_colonize=gui_player():can_colonize(planet) 
          if planet.size=='Tiny'
						str='---'
					elseif (not can_colonize) 
					
						local ship,fleet = light_report_search(gui_player(),type='colonize',star=planet.star) and get_candidate_colonizer(gui_player(),planet)
						if ship
							can_colonize=true
							local race = next | ship.carrying_pop
							str = tostring(planet:species_max(race))
						else
	            str = "--"..str.."--"
						end


          end
          overlays.pop_bar(
            x, str ,color,1,
            nil,nil, (not can_colonize) and  "666666")
        end

      end

      if (el.hover_id==planet) and not FLAG.no_planet_hovers
        local cx=x
        --local cy=bottom_bar.height*.5
        
        local alpha=hover_alpha_states.state[hover_id] or 0

        local dy=45

        local y1=cy
        if planet.size=='Small'
          y1=cy+10
        elseif planet.size=='Medium'
          y1=cy+10
        elseif planet.size=='Large'
          y1=cy-5
        end

        local y2=y1+dy
        local y0=y1-dy


        if planet.defenses>0 and planet.empire==gui_player()
          hover_button(planet, cx, y2, 'gun', 'View Defenses',planet_ship_status)
        end


        if planet.defenses>0 and planet.empire~=gui_player()
          hover_button(planet, cx, y1, 'gun', 'View Defenses',planet_info_click)    
        else
          hover_button(planet, cx, y1, 'question', 'View Planet',planet_info_click)    
        end


        if planet.empire==gui_player() and not planet.no_colony

          local text,can_transport = PlanetTransport.get_planet_tooltip(planet)

          if can_transport
            hover_button(planet, cx, y0, 'spaceship_colored', text,on_planet_transport_button)   
          else
            hover_button(planet, cx, y0, 'spaceship_grey', text,dont_transport_colonists)   
          end


        elseif planet.empire and gui_player():can_contact(planet.empire)
                  

            hover_button(planet, cx, y0, notification_icons.orbital_stuff.talk, 
                  'Contact '..planet.empire.leader.name,contact_faction)   



        end

      
      end

  
    end

    --local can_see_enemy_production=true
    local can_see_enemy_production=false


    local report_act = get_report_actions(planet)

    if report_act and not no_planet_action
      
      report_act(x,planet,fade)

    elseif planet.productive and
      (planet.empire == gui_player() or can_see_enemy_production )
      and not (FLAG.no_planet_hovers)


      local show_star = not planet.player_set_production

        el = element(  SCOPE_call(..planet.show_metal_warning, production_tab,x,
          planet.project_name, fade, 
          show_star, 
          planet:production_progress() ))
      
      
      if planet.productive and planet.empire==gui_player()
        el.planet=planet._ref
        el.on_click=planet_info_click
        el.on_hover=hand_cursor
      else
        el.on_hover=grey_cursor
      end


    end

    x+=.5*(planet_w+spacing)
  end




  -- draw the star title
  if(star and not title_covered() ) and not notitle

    --local s= bh/290*.85
    set_frame_color4(1,1,1,1)

    

    if star_selection_mode

      bottom_bar.title(  star_selection_mode.title?(star) or 'TODO!')

    elseif show_drag_fleet_preview(get_selected_fleet())

      bottom_bar.title(  FleetMove.title?(star) or 'TODO!')

    elseif planet_selection_mode
    
    --transport_src and transport_src.star~=star
      
      bottom_bar.title( planet_selection_mode.title?(star) or 'TODO!')
      
    elseif FLAG.star_element_draw
      

      if star.editable_name and not midscreen_pane

        draw_editable_star_name(star)

      else

				if star.type=='hsa'
					bottom_bar.title | format("Anomaly: %s",star.name)
				else
	        bottom_bar.title(star.name .. ' System')
				end

      end

    end
  end

end


function draw_star_element(star)
  local revealed = revealed_stars()
  if revealed[star]
    FLAG_call( 'star_element_draw', draw_fading_planets, star )
  else
    draw_unknown_star(star)
  end
end


end
reload_script [[GUI\~GalaxyMap\GalaxyMap.lua]]
