

local gzero = generator(0)

function EmpireProperty.has_inhabited_planets(empire)
  for planet in pairs(empire.planets) if planet.inhabited
    return true
  end end
end

function EmpireProperty.base_income(empire)
  return 10
end

function _G.reset_empire_incomes()

	Empire.income = weak_memoizeNN | function (empire)

		if not empire.has_inhabited_planets
			return 0,0,0,0,0,0,0,0
		end

		local taxes, trade, planet_up, ship_up, tank_up, hostile_up in gzero

		for planet ; empire.planets 
      if not planet.no_colony
			  taxes+=planet.coin_production
			  local planet_upkeep, umprovement_up, orbital_up, tank_upkeep, hostile_env=planet:upkeep()
			  planet_up+=planet_upkeep-tank_upkeep-hostile_env
			  tank_up+=tank_upkeep
			  hostile_up+=hostile_env
			  for ship in planet.refit?
				  ship_up+=ship.coin_upkeep
			  end
      else
        for orbit_i, ship ; planet.orbitals
          ship_up+=ship.coin_upkeep
        end
		  end 
    end

		for i,fleet in pairs(empire.fleets) do
			for ship ; fleet.ships
				ship_up += ship.coin_upkeep
        --print(..ship.name,..ship.coin_upkeep)
			end
		end

		-- ships in the trade pool never cost upkeep --
		-- maintenance costs are assumed to be paid by the civilian
		-- captains leasing them.
	--	for i,ship in pairs(empire.tradeship_pool?)
	--		ship_up += ship!.upkeep
	--	end

		for i,fleet in pairs(empire.transports)
			for ship ; fleet.ships
				ship_up += ship!.coin_upkeep
			end
		end

		for i,fleet in pairs(empire.recalled_ships)
			for ship ; fleet.ships
				ship_up += ship!.coin_upkeep
			end
		end

		--console(..taxes,..ship_up,..planet_up)

		local trade = min(empire.active_trade, empire.trade_capacity)
		local foreign_trade_bonus = 1*min(trade, empire.total_foriegn_trade_routes)
		trade+=foreign_trade_bonus

		local net_agreement = empire : net_agreement_income()

		local food_bonus = max(0,floor | .25*empire.net_food_production)

		return empire.base_income + net_agreement + taxes + trade - ship_up - planet_up+food_bonus-tank_up-hostile_up,
			taxes, trade, ship_up, planet_up, net_agreement,foreign_trade_bonus,food_bonus,tank_up,hostile_up

	end

end

_G.reset_empire_incomes()

function Fleet.scuttle(fleet,ship)

  if fleet.pinned_by_attack
    error "its illegal to scuttle a ship in a pinned fleet."
  end


	ship.dead=true
	remove_ship(ship,fleet)
	if not next | fleet.ships
		remove_fleet(fleet)
	end

	reset_empire_incomes()
end

function EmpireProperty.scrap_mult(empire)
	return .05
end

function EmpireProperty.scrap_metal_mult(empire)
	return .5
end

function FleetProperty.pinned_by_attack(fleet)

  local star = at_star(fleet) 

  return pending_attacks[star] and fleet_is_pinned(fleet.empire,star)

end


function ShipProperty.cant_scuttle(ship)
	local fleet = find_ships_fleet(ship)

  if fleet
		if fleet.pinned_by_attack
      return 'under_attack'
    end

    return

  elseif ship.empire.tradeship_pool[ship]
    return
  else

    local planet = ship:find_orbital()

    if planet
      if pending_attacks[planet.star]?.planet==planet
        return 'under_attack'
      end
    end
  end
end

function Empire.allied_control(empire,star)

  if empire.stars[star]
    return empire
  end
  for i,e in voting_empires()
    if alliance(e,empire) and e.stars[star]
      return e
    end
  end

end

function ShipProperty.cant_scrap(ship)
  local fleet = find_ships_fleet(ship)

  if ship.mercenary_empire
    return 'mercenary'
  end

	if fleet
		local star = at_star(fleet)

    if fleet.pinned_by_attack
      return 'under_attack'
    end

    if not( star and ship.empire:allied_control(star) )
      return 'not_at_owned_star'
    end
    return
	end

  local planet = ship:find_orbital()
  if planet
    return ship.cant_scuttle
  end

  return 'not_at_owned_star'
end

function ShipProperty.cant_instant_refit(ship)
  local cant_scrap in ship
  if  cant_scrap return cant_scrap end

  if ship.refitting_at
    return 'refitting_at_planet'
  end

  local fleet = find_ships_fleet(ship)
  if fleet
    local star=at_star(fleet)!
    -- make sure there's a planet in the system with sufficient
    -- facilities to do the refit
    local planet_cant
    for slot,planet in star:planet_list()
      if planet.empire and ( (planet.empire==ship.empire or alliance(planet.empire,ship.empire)) and (not planet.cant_refit_any) )
        local cant = planet:cant_refit(ship)
        if not cant
          return
        else
          planet_cant=cant
        end
      end
    end


    return planet_cant or 'no_colony'
  end
    
  local planet = ship:find_orbital()
  if planet
    if planet.no_colony
      return 'no_colony'
    end
  end

end

function ShipProperty.can_scuttle(ship)
  return not ship.cant_scuttle
end


function ShipProperty.can_scrap(ship)
  return not ship.cant_scrap
end

function ShipProperty.scrap_value(ship)
	return round | (ship.hammer_cost* ship.empire.scrap_mult )
end

function ShipProperty.scrap_metal(ship)
  return round | (ship.metal_cost* ship.empire.scrap_metal_mult )
end

--function 

function Fleet.scrap(fleet,ship)
	ship.dead=true
	remove_ship(ship,fleet)
	if not next | fleet.ships
		remove_fleet(fleet)
	end
	reset_empire_incomes()
	local scrap_value,scrap_metal in ship
	ship.empire.credits+=scrap_value
  ship.empire.metal+=scrap_metal
	return scrap_value,scrap_metal
end

function check_for_lost_outpost(planet)

  --print ('checking for lost outpost case on',planet.name,planet.outpost, planet.no_colony)

  if (planet.no_colony) and (not planet.outpost)
    local old_empire=planet.empire
    planet.empire=nil
    planet.no_colony=nil
    print('removing outpost from',planet)
    note_planet_transfer(planet,old_empire)
    remove_invalid_orbit_reports()
    update_star_reports(planet.star)
  end

end

function Planet.scrap_orbital(planet,i)
	local ship = planet.orbitals![i]
	planet.orbitals[i]=nil
	ship.dead=true
	local scrap_value = round | (ship.hammer_cost* ship.empire.scrap_mult )
	local scrap_metal = round | (ship.metal_cost* ship.empire.scrap_metal_mult )
	--console(..scrap_value )
	ship.empire.credits+=scrap_value
  ship.empire.metal+=scrap_metal

  local empire in ship
	reset_empire_incomes()
  check_for_lost_outpost(planet)
  update_stargate_reports_for_star(empire,planet.star)
	return scrap_value
end


function ShipMethod.find_orbital (ship)
		
	for planet ; ship.empire?.planets?
		for i,s ; planet.orbitals?
			if s==ship
				return planet,i
			end
		end
	end

end

function order.scrap_ship(ship)
	local fleet = find_ships_fleet(ship)



  ship:destroy_any_refit_references()

	if fleet

    local star = at_star(fleet)

    local can_scrap in ship

		if can_scrap
			fleet:scrap(ship)
		else
			fleet:scuttle(ship)
		end

    if star
      remove_invalid_orbit_reports()
      update_star_reports(star)
    end

	elseif ship.is_fixed 
    if ship:find_orbital()
		  local planet, orbital = ship:find_orbital()
		  planet:scrap_orbital(orbital)
    else
      local planet = ship!.orbiting_planet
      for i,s ; planet.orbitals?
			  if s==ship
				  planet:scrap_orbital(i)
			  end
		  end     
    end
	elseif ship.empire.tradeship_pool?[ship]
    ship.dead=true
    ship.empire.tradeship_pool[ship]=nil
  else

    -- should only be necessary in games started prior to r12019
    local planet, orbital = ship:find_orbital()
    if planet
      planet:scrap_orbital(orbital)
      return
    end

		trace_error("don't know how to scrap this ship",1,5)
	end


end

	function _G.ship2planet (ship)
		local empire in ship
		for planet ; empire.planets
			for i,s ; planet.orbitals?
				if s==ship
					return planet,i
				end
			end
		end

	end


function handle_deficit(empire)



	-- first, try scrapping ships
	local scrap_list = {}

	local debt= -(empire.credits + empire:income() )

	local function new_deficit_report()
		add_report {
			type='deficit',
			..empire,
			..scrap_list,
			..debt,
			abandon_list={},
		}
	end

	local push, scrappable = pusher {}

	for i,fleet ; empire.fleets
		local star=at_star(fleet)
		for ship ; fleet.ships
			push { ..ship, ..fleet, ..star, ..fleet.eta}
		end
	end

	table.sort(scrappable,|) function(a,b)
		return lexical_compare(
			b.eta or 0, a.eta or 0,
			a.ship._ikey,b.ship._ikey
		)
	end

	for _,info in ipairs | scrappable 				
		local star,fleet,ship in info

		local v = {..ship, type='ship', ..star, ..value,..fleet,..fleet.eta,..fleet.dest_star,..fleet.src_star }
		v.value = FLAG_call('handling_deficit',fleet.scrap,fleet,ship)
		append(v,scrap_list)
	
		if (empire.credits + empire:income() ) >= 0
			return new_deficit_report() 
		end	

	end

	return new_deficit_report()

end


economy_update = flame_fun('economy_update',|)function()

  --
  -- for each empire, first:
  --
  -- 0) move trade ships into the pool.

  -- 1) apply metal /breaker production for all planets
  -- 2) process production for all planets
  --
  -- 3) apply_morale_effects(). 
  --
  -- 4a) farming: calculate the starvation state.
   --4b) farming: update each planet's population to account for growth/starvation.
  --
  -- 5) complete research projects
  
  -- 6) do ship motion
  --
  -- note that this means that the galaxy state in which the starvation rules
  -- are applied may be significantly different than the galaxy state at the
  -- end of the year.  buildings with eta 1 will have completed, and trade 
  -- ships with eta 1: will have finished moving into the pool.
  --
  -- however, military fleets will not yet have moved, thus, the blockade
  -- state of any planets will not have changed.  similarly, research
  -- will not have completed, thus, any techs that might effect food
  -- production won't yet be applied.
  --


	for _,empire in pairs(all_empires) 
		for ship,info ; empire.tradeship_pool?
			if info.eta>0
				info.eta-=1
			end
		end
	end


	for _,empire ; all_empires if not empire.monster_economy

		if empire.credits + empire:income() < 0
			handle_deficit(empire)
		end

		empire.credits+=empire:income()	
	end end


	for _,empire in pairs(all_empires) if not empire.monster_economy

    if not empire.researching
      on_no_research(empire) 
    end

    for planet ; empire.planets if not planet.no_colony
      empire.metal+=planet.metal_production
    end end

		--empire.science+= empire : research()
    empire:gain_beakers( empire : research() )
    order.remove_starving_city_planning(empire)
    reset_food_caches()

    -- the order in which planet's production is updated can
    -- matter, as planet's later in the queue are more likely to be 
    -- stalled due to lack of metal.
		for planet in rpairs(empire.planets,|) 
      function(p1,p2)
        return lexical_compare(p2.raw_hammer_production,p1.raw_hammer_production, recorded_name(p1), recorded_name(p2) )
      end


--    if planet.inhabited 
      if planet.inhabited or planet.productive --jem.habitat.build, allow orbital colonies to build stuff
			if(planet.skip_production)
				planet.skip_production=nil
--			elseif not planet.no_colony
			else --jem.habitat.build
				production_update(planet)
				change_production_if_needed(planet)	
        planet:create_overflow_project()
			end
		end end

    update_empire_food(empire)

    empire : research_turn_complete()

	end end

  -- note that morale is not applied to empires with monster
  -- economies.  this means that no matter how miserable life is
  -- on a planet, once it forms a new empire by revolting,
  -- population under the rebel government will never become
  -- discontent, and/or revolt.
	for _,empire in pairs(all_empires) if not empire.monster_economy
     local _TRACE_INFO=empire.name
     for planet ; empire.planets
      if not planet.no_colony
        planet : apply_morale_effects()
      end
    end   
  end end

  monster_economy_update()

  for _,empire in pairs(all_empires)
    update_influence | empire
  end
end
