local max,min,ceil,floor in math

local project_complete,project_cost = {}, {}, {}


local calc_cost = {}

function calc_cost.building_improvement(planet)
  return planet:improvement_cost(planet.building_improvement)
end

local get_design_cost = weak_memoizeNN | function(empire,production)
	local design = all_designs![production]
  local _TRACE_INFO=empire.name
  assert(design.empire==empire)
	--local _TRACE_INFO = arg_list(..production,..design.config_name,..design.ship_drawer)
	local cost_vector = catch ( get_cost_values | design.update_hull_specs() )

  if not empire : known 'Subspace Networking'
    return cost_vector()
  else
    local hammer_cost = cost_vector()
    return ceil(hammer_cost*.75) , drop_args(1,cost_vector())
  end


end

function calc_cost.refitting(planet)
  return planet:refit_cost(planet!.refitting)
end

function calc_cost.building_ground_unit(planet)
  if planet.building_ground_unit=='tank'
    return planet:tank_cost()
  elseif planet.building_ground_unit=='android'
    return planet:android_cost()
  elseif planet.building_ground_unit : match('raider')
    return planet:raider_cost()
  end
  error 'FIX THIS!'
end


function calc_cost.refitting_orbital(planet)
  return planet:refit_orbital_cost(planet!.refitting_orbital)
end

function Planet.colony_base_cost(planet,nplanet)
  return production_cost_adjust | 380
end

function Planet.build_planet_cost(planet)
  return production_cost_adjust | 1500
end

function calc_cost.building_colony(planet)
  return planet:colony_base_cost(planet.building_colony)
end

function calc_cost.building_planet(planet)
  return planet:build_planet_cost(planet)
end

function calc_cost.terraforming(planet)
  return planet : can_terraform( planet!.terraforming )
end

function calc_cost.building_design(planet)
--  return planet:improvement_cost(planet.building_improvement)
  return planet:design_cost(planet.building_design)

end

calc_cost.activity = _SAFE


function PlanetProperty.current_production_cost(planet)
  for _,k in ipairs | project_type_list
    if planet[k]
      return  calc_cost[k](planet)
    end
  end
end

local set_project_orders = {
  set_building_design = 'building_design', 
  set_activity = 'activity',
  set_building_improvement='building_improvement',
  set_terraforming='terraforming',
  set_building_colony='building_colony',
  set_building_planet='building_planet',

  start_orbital_refit='refitting_orbital',
  start_refit='refitting',
  
  set_building_ground_unit = 'building_ground_unit',
}

metal_project_costs = {
  building_design = 'design_cost', 
  refitting_orbital = 'refit_orbital_cost', 
  refitting = 'refit_cost', 
  building_ground_unit = 'ground_unit_cost', 
}

_G.project_field_to_order = invert_table(set_project_orders)

function _G.undo_set_project(planet)
  --temporarily disabling this.
  do return end

  local last_order,empire,args = last_order_info()

  if set_project_orders[last_order] and empire==gui_player() 

    local planet_name=args:match( "..(.*)%._ref")

    if(planet_name==planet._recorded_name)
      undo_order()
    end

  end

end


function Planet.ship_build_discount(planet)

	local ship_cost_mult=1

	for i,station ; planet.orbitals?
		ship_cost_mult = min(ship_cost_mult,(station.ship_build_discount or 1))
	end

	return ship_cost_mult * (planet.empire.global_ship_cost_mult or 1)
end


function PlanetProperty.active_project_discount(planet)

  if planet.building_design or planet.refitting or planet.refitting_orbital
    return planet:ship_build_discount()
  end

  return 1
end



function buy_production(planet)

	if planet_blockaded(planet)
		trace_error("can't buy production on a blockaded planet")
	end

	local cost,hammers,metal = planet:buy_cost()

	local empire in planet

	if(cost>empire.credits)
		trace_error("not enough coins to buy production",2,4)
	end

  if metal > empire.metal
 		trace_error("not enough metal to buy production",2,4)   
  end


	empire.credits-=cost
	planet.hammers+=hammers
  planet.building_metal=planet.building_metal+metal
  empire.metal-=metal

	planet.fixed_production=true
	planet.skip_production=nil

  if planet.building_improvement=='Farm'
    remove_starvation_warning_if_needed(planet.empire)
  end
end



do


  local function cost_estimate(planet,cost)

    local hammer_production in planet

    if hammer_production==0
      return max_double,cost
    end


    return  ceil | max(0,  (cost-planet.hammers-planet.overflow_hammers)/planet.hammer_production), cost
  end


  local function current_production_estimate(planet)
    local cost = planet.current_production_cost
    return cost_estimate(planet,cost)
  end


  local estimate = {
    building_improvement = current_production_estimate,
    building_design = current_production_estimate,
    building_ground_unit = current_production_estimate,
    building_colony = current_production_estimate,
    building_planet = current_production_estimate,
    refitting = current_production_estimate,
    refitting_orbital = current_production_estimate,
    terraforming = current_production_estimate,
  }


  function Planet.design_cost(planet,design_key)
    return planet:ship_discount_cost(  get_design_cost(planet.empire,design_key) ) 
  end


  function Planet.turns_to_complete(planet)
    for _,k in ipairs | project_type_list
      if planet[k]
        return estimate[k]?(planet)
      end
    end
  end
end

function EmpireProperty.coins_to_labor_rate(empire)
  return empire : known 'Industrial Replicators' and 1 or 3
end

function Planet.buy_cost(planet)
  local _TRACE_INFO=production
	local hammer_cost,metal_cost 

  local project,info = planet : project_info()

  local metal_fun = metal_project_costs[project]

  --print(..metal_fun,..project)

  if metal_fun
    hammer_cost,metal_cost = planet[metal_fun](planet,info)    
  else
    hammer_cost = planet.current_production_cost
  end

	local remaining = hammer_cost-planet.hammers-planet.overflow_hammers
  remaining=max(0,remaining)
	return 
    remaining * planet.empire.coins_to_labor_rate ,
    remaining,
    max(0, (metal_cost or 0) - planet.building_metal)
end



function Planet.production_progress(planet)

	if planet.activity

		if planet.activity=="City Planning" or planet.activity=='Manufacture Population'
			local a,b=planet:growth_progress()
			return a,b,true
		end

    if planet.activity=='Research'
      return research_progress(planet.empire)
    end


		return nil
	end

  local cost = planet.current_production_cost

	if(not cost) return 0 end

  local next_hammers = planet.hammers + planet.hammer_production + planet.overflow_hammers

	return planet.hammers / cost, min(1, next_hammers / cost)

end

function Empire.science_station_research(empire)
  local research=0

  -- atm, ships under civilian command don't generate
  -- research.  but, space stations do.
	for _,fleet ; empire.fleets
    for ship ; fleet.ships
      research+=ship.beaker_gen or 0
    end
  end

	for planet ;empire.planets
    for _,orbital ; planet.orbitals?
      research+=orbital.beaker_gen or 0
    end 
	end
    
  return research
end

function Empire.raw_research(empire)

  if empire.eliminated
    return 0
  end

	local research=0

	for planet in pairs(empire.planets) do
		research+=planet.beaker_production
	end

  research += empire:science_station_research()

  return research
end

function Empire.research(empire)
  local research = empire : raw_research()

  research += empire!.scientist?.beaker_production or 0
  research += empire:total_research_agreement_beakers()

	return research
end

Empire.mem_research = weak_memoizeNN | Empire.research


function PlanetProperty.excess_hammers(planet)
  return 0
end

function PlanetProperty.overflow_hammers(planet)
  if not planet.overflow
    return 0
  end
  local year in galaxy
  return (planet.overflow[year] or 0) + (planet.overflow[year-1] or 0)
end

function Planet.clear_overflow_hammers(planet)
  for k ; planet.overflow?
    planet.overflow[k]=nil
  end
end

function Planet.gain_overflow_hammers(planet,hammers)
  if not planet.overflow
    planet.overflow={}
  end
  local year in galaxy
  planet.overflow[year] = (planet.overflow[year] or 0) + hammers
end

function Planet.use_overflow_hammers(planet,hammers)
  local year in galaxy
  local y0 = year-1
  
  assert(hammers <= planet.overflow_hammers)

  if planet.overflow[y0]
    if hammers <= planet.overflow[y0]
      planet.overflow[y0]-=hammers
      return
    else
      hammers-=planet.overflow[y0]
      planet.overflow[y0]=nil
    end
  end

  planet.overflow[year]-=hammers
end

-- just before it's due to expire, move last-year's overflow
-- hammers into a 'incomplete project', so that they can
-- potentially be accessed by the AI when using 
-- the 'pool_resources' option.
function Planet.create_overflow_project(planet)
  local year in galaxy
  local y0 = year-1
  if not planet.overflow?[y0]
    return
  end

  if not planet.incomplete_projects 
    planet.incomplete_projects = {}
  end

  if not planet.incomplete_projects.overflow_project
    planet.incomplete_projects.overflow_project={}
  end

  planet.incomplete_projects.overflow_project.hammers =  planet.overflow[y0] +
    (planet.incomplete_projects.overflow_project.hammers or 0)
  planet.overflow[y0]=nil
end


function clear_current_project(planet)

  -- don't carry over resources from prior projects
  planet.hammers=nil
  planet.building_metal=nil

  for _,k in ipairs | project_type_list
    planet[k]=nil
  end

end

-- ok.
-- so when we switch from one planet project to another, the
-- current project is "deactivated" rather than "cleared". 
-- assuming there's some storable progress in the current
-- project, we store it in the incomplete_projects
-- table.
--
-- what, exactly, this means varies depending on the type of
-- project being stored:
--
-- building_improvement projects store only hammers, which will
-- be recovered we switch back to the same buiding_improvement
-- type.
--
-- building_colony projects store hammers, which will be
-- recovered if we switch back to *any* building_colony
-- project.
--
-- refitting and refitting_orbital projects store hammers,
-- which will be recovered if we switch to *any* new refit
-- project.
--
-- terraforming projects store hammers, which will be recovered
-- if we switch to *any* terraforming project.
--
-- building_design projects store both hammers and metal, which
-- will be recovered if we switch to any building design
-- project that matches the config_name assosiated with the old
-- design.  this is the most subtle case -- and rests on the
-- assumption that all ships with the same config name will
-- have the same metal cost (though not, necessarily, the same
-- hammer cost).
--

local refit_projects = set('refitting','refitting_orbital')

function recover_refit_hammers(planet)
  for i,v ; planet.incomplete_projects?
    if v.refitting
      planet.hammers=v.hammers
      planet.incomplete_projects[i]=nil
      return
    end
  end
end

function deactivate_current_project(planet)
  reset_food_caches()
  
  local project_type, info = planet: project_info()

  -- activities are never stored.
  if project_type=='activity'
    local old_activity=planet.activity

    clear_current_project(planet)

    if old_activity=='City Planning'
      remove_starvation_warning_if_needed(planet.empire)
    elseif old_activity=='Synthesize Food'
      add_starvation_warning_if_needed(planet.empire) 
    end

    return
  end

  if project_type
    if not planet.incomplete_projects 
      planet.incomplete_projects = {}
    end
    local hammers, building_metal, incomplete_projects in planet

    -- don't store incomplete production if there's nothing to
    -- store.

    --print('switching from',project_type, info)

    if planet.producing_hammers and hammers==0
      return clear_current_project(planet)
    end

    local v

    do
      v = { 
        [project_type] = info,
        ..hammers,
      }

      if (project_type== 'building_ground_unit') or refit_projects[project_type]
        -- hrm.  should we change this?
        insert(v,..building_metal)
      elseif project_type=='building_design'
        v.config_name =all_designs![info]!.config_name
        v.ship_drawer =all_designs![info]!.ship_drawer
        insert(v,..building_metal)
      end
    end

    --print('adding project',v2s | v)

    incomplete_projects[#incomplete_projects+1] = v

    planet.hammers=nil
    planet.building_metal=nil
    planet[project_type]=nil
  end

  if project_type=='building_improvement' and info =='Farm'
    add_starvation_warning_if_needed(planet.empire)
  end

end

do


  function StarProperty.cant_build_more_planets(star)
    local a,b=StarOrbitRange![star.type]()
    for i=a,b
      if not star.planets[i]
        return false
      end
--      if (star.planets[i].type=='Gas Giant') and (not star.planets[i].empire)
      if (star.planets[i].type=='Gas Giant' or star.planets[i].type=='Inferno' or star.planets[i].size=='Tiny' or star.planets[i].size=='Small' or star.planets[i].temperature=='Cold' or star.planets[i].minerals=='Poor') and (star.planets[i].no_colony or not star.planets[i].empire) --jem
        return false
      end
    end

    return true
  end

    function StarProperty.num_planet_build_sites(star)
    local a,b=StarOrbitRange![star.type]()
    local rval=0
    for i=a,b
      if not star.planets[i]
        rval+=1
--      elseif (star.planets[i].type=='Gas Giant') and (not star.planets[i].empire)
      elseif (star.planets[i].type=='Gas Giant' or star.planets[i].type=='Inferno' or star.planets[i].size=='Tiny' or star.planets[i].size=='Small' or star.planets[i].temperature=='Cold' or star.planets[i].minerals=='Poor') and (star.planets[i].no_colony or not star.planets[i].empire) --jem
        rval+=1
      end
    end

    return rval
  end


  _G.building_planet_request_stub = weak_memoizeNN | function()
    return EmulateRecorded.all_planets {
      size='Large',
      type='Barren',
      day_length=32,
      orbit=2,
    }
  end

  _G.terraforming_planet_request_stub = weak_memoizeNN | function(type,size)
    return EmulateRecorded.all_planets {
      ..type,
      ..size,
      day_length=32,
      orbit=2,
      pop ={ teros=10, orthin=10, phidi=10 }
    }
  end

  function complete_build_planet_project(planet,report)

    clear_current_project(planet)
    local push, candidate_slots = pusher {}
    local star in planet
    
    local a,b=StarOrbitRange![star.type]()
    for i=a,b
      if (not star.planets[i]) or ( 
--          (star.planets[i].type=='Gas Giant') and (not star.planets[i].empire) )
          (star.planets[i].type=='Gas Giant' or star.planets[i].type=='Inferno' or star.planets[i].size=='Tiny' or star.planets[i].size=='Small' or star.planets[i].temperature=='Cold' or star.planets[i].minerals=='Poor') and (star.planets[i].no_colony or not star.planets[i].empire) ) --jem
        push | i
      end
    end

    --print(v2s | star.planets)

    if not next | candidate_slots
      report.unable_to_complete='No Build Sites'
      return
    end

    local orbit = candidate_slots[ dice_roll(#candidate_slots) ]
    local old_planet = star.planets[orbit]
    local size='Medium'
--    if old_planet
    if old_planet and (star.planets[orbit].type=='Gas Giant' or star.planets[orbit].size=='Large') --jem
      size='Large'
    end

    local kW_m2=1.2

    local new_planet = new_planet_ref {
      type='Barren',
      minerals='Normal',
      ..size,
      ..planet.star,
      ..kW_m2,
      ..orbit,
      day_length=32,
      rarity='common',
      temperature = get_temperature(kW_m2),
      --name = 'Artificial Planet' ,  
			 name = (star.name .. " " .. to_roman(orbit) .. '-A'),
      specials = {}, 
    }
    if old_planet
      if star.planets[orbit].minerals=='Rich' --jem
        new_planet.minerals='Rich'
      end
      if star.planets[orbit].special_resources --jem
	if not new_planet.special_resources
	  new_planet.special_resources={}
	end
        for i,r in ipairs | star.planets[orbit].special_resources?
	  new_planet.special_resources[1+#new_planet.special_resources]=r
        end
      end
    end

    local i=orbit
    star.planets[i]=new_planet

    add_rotation_fluff | new_planet
    add_distance_fluff (star, new_planet )


  --  get_colonist_species

  --  order.init_colony(planet.empire,building_colony,{})
  --  order.colony_base_autoload(planet,building_colony)

  --  order.create_new_colony_reports(building_colony)

  --  check_new_colony_achievements(building_colony)
  end

end

-- compatibility pattern
function PlanetProperty.specials(planet)
  return _SAFE
end

function on_production_complete(planet)
  --clear_current_project(planet)
  --local production_queue in planet

  local in_auto_mode = not planet.player_set_production

  move_to_next_in_queue(planet)

  if (in_auto_mode) and not planet.project_type
    FLAG_call('dont_delete_reports',
      order.set_autobuild,planet)   
  end

  FLAG_call('dont_produce',production_update,planet)

end

function set_building_colony(planet,colony_planet)
   deactivate_current_project(planet) 
   planet.player_set_production=check_flag 'player_set_production' or nil

  for i,v ; planet.incomplete_projects?
    if v.building_colony==colony_planet
      --use this prior project
      planet.hammers = v.hammers
      planet.incomplete_projects[i]=nil
      break
    end
  end

   planet.building_colony=colony_planet
   check_valid_project(planet)
   remove_construction_report(planet)
end


function Planet.stored_hammers_for__building_colony(planet,colony_planet)
  if planet.building_colony==colony_planet
    return planet.hammers
  end
  for i,v ; planet.incomplete_projects?
    if v.building_colony==colony_planet
      return v.hammers
    end
  end 
end

function Planet.stored_progress_for__building_colony(planet,colony_planet)
  local hammers = planet : stored_hammers_for__building_colony(colony_planet)
  if not hammers return end

  local hammer_cost = planet:colony_base_cost(planet,colony_planet)
  local overflow_hammers,hammer_production in planet
  return 
    min( 1 , hammers/hammer_cost), 
    min(1, (hammers+hammer_production+overflow_hammers)/hammer_cost )
end


function set_building_planet(planet,colony_planet)
   deactivate_current_project(planet) 
   planet.player_set_production=check_flag 'player_set_production' or nil

  -- building colony projects can always share stored hammers
  -- with each other.
  for i,v ; planet.incomplete_projects?
    if v.building_planet
      --use this prior project
      planet.hammers = v.hammers
      planet.incomplete_projects[i]=nil
      break
    end
  end

   planet.building_planet=true
   check_valid_project(planet)
   remove_construction_report(planet)
end

function Planet.stored_hammers_for__building_planet(planet)
  if planet.building_planet
    return planet.hammers
  end
  for i,v ; planet.incomplete_projects?
    if v.building_planet
      return v.hammers
    end
  end
end


function Planet.stored_progress_for__building_planet(planet)
  local hammers = planet : stored_hammers_for__building_planet()
  if not hammers return end

  local hammer_cost = planet:build_planet_cost(planet)
  local overflow_hammers,hammer_production in planet
  return 
    min( 1 , hammers/hammer_cost), 
    min(1, (hammers+hammer_production+overflow_hammers)/hammer_cost )
end

function Planet.building_improvement_progress(planet,improvement)

  if planet.building_improvement==improvement
    return planet.hammers
  end

  for i,v ; planet.incomplete_projects?
    if v.building_improvement==improvement
      --use this prior project
      return v.hammers
    end
  end

end

function Planet.stored_progress_for__building_improvement(planet,improvement)
  local hammers = planet : building_improvement_progress(improvement)
  if not hammers return end

  local hammer_cost = planet:improvement_cost(improvement)
  local overflow_hammers,hammer_production in planet
  return 
    min( 1 , hammers/hammer_cost), 
    min(1, (hammers+hammer_production+overflow_hammers)/hammer_cost )
end

function order.set_building_improvement(planet,improvement,option)
 	local old_activity=planet.activity 
	if planet.building_improvement==improvement
    planet.player_set_production=check_flag 'player_set_production' or nil
    return
  end

  deactivate_current_project(planet)

  for i,v ; planet.incomplete_projects?
    if (v.building_improvement==improvement) or (option=='pool_resources')
      --use this prior project
      planet.hammers += v.hammers
      planet.empire.metal += (v.building_metal or 0) -- (should only matter for pooled resources)
      planet.incomplete_projects[i]=nil
    end
  end
  
  planet.player_set_production=check_flag 'player_set_production' or nil
  planet.building_improvement = improvement!

  if (planet.building_improvement=='Farm')
    remove_starvation_warning_if_needed(planet.empire)
  end

  check_valid_project(planet)
  remove_construction_report(planet)

	if old_activity=='Trade'
		reset_empire_incomes()
	end
end

function order.set_activity(planet,activity)
	local old_activity=planet.activity
  deactivate_current_project(planet)
  planet.player_set_production=check_flag 'player_set_production' or nil
  planet.activity = activity!
  remove_construction_report(planet)
  if planet:invalid_project(planet) == 'starving'
    return
  end
  check_valid_project(planet)  

	if (planet.activity=='Trade') or old_activity=='Trade'
		reset_empire_incomes()
	end

  if planet.activity=='Synthesize Food'
    remove_starvation_warning_if_needed(planet.empire)
  end
end

function remove_construction_report(planet)
  if FLAG.dont_delete_reports
    return
  end

  local r = light_report_search(planet.empire,type='project_changed',..planet)
  if r
    r.unlisted=true 
  end
  local r = light_report_search(planet.empire,type='construction',..planet)
  if r
    r.unlisted=true 
  end
  local r = light_report_search(planet.empire,type='new_colony',..planet)
  if r
    r.unlisted=true 
  end


end

local do_process = {}

local function production_complete_handler(fun)

  return function(planet)

    local building_design,empire,hammers,building_metal,hammer_production in planet

    local cost = planet.current_production_cost

    -- use both our excess hammers and our production.
    --planet.hammers += hammer_production+planet.excess_hammers
    --planet.excess_hammers=nil

    if cost <= (planet.hammers + hammer_production + planet.overflow_hammers)
    
      local new_overflow = planet.hammers + hammer_production - cost
      if new_overflow > 0
        planet:gain_overflow_hammers(new_overflow)
      elseif new_overflow~=0
        planet:use_overflow_hammers(-new_overflow)
      end

      planet.hammers=nil

      local report

      if FLAG.dont_produce

        report = light_report_search(empire,type='construction',..planet)
        report.completed_projects[#report.completed_projects+1] = {
     		  completed_project=planet.project_name,
          completed_project_type=planet.project_type,  
        }

      else

        -- this needs some cleverness in the FLAG.dont_produce case
	      report = add_report {
		      type="construction",
		      ..planet,
		      ..planet.empire,
		      completed_project=planet.project_name,
          completed_project_type=planet.project_type,
          completed_projects = {
            {
     		      completed_project=planet.project_name,
              completed_project_type=planet.project_type,       
            },
          }
	      }

      end

      fun(planet,report)
      on_production_complete(planet)
    else
      planet.hammers += hammer_production + planet.overflow_hammers
      planet : clear_overflow_hammers()
    end

  end

end


function complete_build_colony_project(planet,report)
  local building_colony in planet

  clear_current_project(planet)

--  get_colonist_species

  order.init_colony(planet.empire,building_colony,{})
  order.colony_base_autoload(planet,building_colony)

  order.create_new_colony_reports(building_colony)

  check_new_colony_achievements(building_colony)
end

do_process.building_colony = production_complete_handler | function(...)
  order.complete_build_colony_project(...)
end



do_process.building_planet = production_complete_handler | function(...)
  order.complete_build_planet_project(...)
end

local improvement_achievement_stat = {
  factories = 'MOST_FACTORIES_ON_PLANET',

  labs = 'MOST_LABS_ON_PLANET',
  mines = 'MOST_MINES_ON_PLANET',

  farms = 'MOST_FARMS_ON_PLANET',
  markets = 'MOST_MARKETS_ON_PLANET',
}

do_process.building_improvement = production_complete_handler | function(planet)

  local improvement = planet.building_improvement
  local field = planet_improvement_to_field![improvement]

  planet[field] =  planet[ field ]+1

  local empire in planet
  if empire==gui_player()
    local stat = improvement_achievement_stat[field]
    if stat
      set_achievement_increment_only_int(stat,planet[field])
    end

    if not empire.total_improvements_built
      empire.total_improvements_built=1
    else
      empire.total_improvements_built++
    end
    set_achievement_increment_only_int('MOST_IMPROVEMENTS_BUILT',empire.total_improvements_built)
  end

	order.check_connected_tinkers(planet)

end

do_process.building_design =  function(...)
  process_building_design(...)
end

do_process.building_ground_unit =  function(...)
  process_building_ground_unit(...)
end


do_process.refitting = function(...)
  process_refitting(...)
end

do_process.refitting_orbital =  function(...)
  process_refitting_orbital (...)
end

--[[ 
do_process.refitting = production_complete_handler | function(...)
  order.complete_refit_project(...)
end


do_process.refitting_orbital = production_complete_handler | function(...)
  order.complete_refit_orbital_project(...)
end

--]]

do_process.activity = function(...)
  order.activity_update(...)
end

do_process.terraforming = production_complete_handler | function(...)
  order.complete_terraforming_project(...)
end

function production_update(planet)

	planet.fixed_production=nil

  if (planet.no_production_set) and (not FLAG.dont_produce)
    move_to_next_in_queue(planet)
    if planet.no_production_set
      set_autobuild(planet)
    end
  end

  for _,k in ipairs | project_type_list
    if planet[k]
      do_process![k](planet)
      break
    end
  end


end







